#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include <sys/time.h>

#include "config.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xmd.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xatom.h>
#include <X11/Shell.h>
#include <X11/Xaw/XawInit.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Scrollbar.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/Box.h>


#include "xawdecode.h"
#include "strtab.h"
#include "channel.h"
#include "grab.h"
#include "mixer.h"
#include "x11.h"
#include "toolbox.h"
#include "crypt.h"
#include "nagra.h"
#include "const.h"
#include "wintv_colors.h"
#include "nagraudio.h"
#include "filtre.h"
#include "kizkool.h"
#include "fasouil.h"
#include "plugin.h"
#include "plugin_ui.h"

XtActionsRec actionTable2[] = {
  {"Plugin", PluginAction},
  {"Nagraudio", NagraudioAction},
  {"Quality", HighQualityAction},
  {"FPS", FPSAction},
  {"StatusLine", StatusLineAction},
  {"IncCouple", CoupleIncAction},
  {"DecCouple", CoupleDecAction},
  {"CryptMode", CryptModeAction},
  {"SoundMode", SoundModeAction},
  {"CryptModedirekt", CryptModeActiondirekt},
  {"SetBurst", SetBurstAction},
  {"Decode", DecodeAction},
#ifndef DISABLE_ALL_FILTERS
  {"BlueFilter", BlueFilterAction},
  {"BlueFilterToggle", BlueFilterToggleAction},
# ifndef DISABLE_ALL_KIZKOOL
   {"SmudgeFilterToggle", SmudgeFilterToggleAction},
   {"SmudgeFilterPhaseToggle", SmudgeFilterPhaseToggleAction},
   {"FilterThreshold", FilterThresholdAction},
# endif
#endif
  {"SetBlack", SetBlackAction}
};

static struct STRTAB couples_list[] = {
  { 4, " 4"},
  { 8, " 8"},
  {12, "12"},
  {16, "16"},
  {20, "20"},
  {24, "24"},
  {28, "28"},
  {32, "32"},
  {40, "40"},
  {48, "48"},
  {56, "56"},
  {64, "64"},
  {72, "72"},
  {80, "80"},
  {88, "88"},
  {-1, NULL}
};

static struct STRTAB decode_list[] = {
  {CLEAR, "CLEAR"},
  {NAGRA, "Nagra"},
  {VCRYPT, "VCRYPT"},
  {SCRYPT, "SCRPYT"},
  {HCRYPT, "HCRYPT"},
  {XCRYPT, "XCRYPT"},
  {-1, NULL,}
};

static struct STRTAB sound_list[] = {
  {NAGRAUDIO_OFF, "CLEAR"},
  {NAGRAUDIO_ON, "Nagra"},
  {NAGRAUDIO_AUTO, "Auto"},
  {-1, NULL,}
};

static struct STRTAB nagraudio_auto_list[] = {
  {1, "On"},
  {0, "Off"},
  {-1, NULL,}
};

static struct STRTAB nagraudio_filter_list[] = {
  {0, "Off"},
  {1, " 1"},
  {2, " 3"},
  {3, " 5"},
  {4, " 7"},
  {-1, NULL,}
};

static struct STRTAB nagraudio_gain_list[] = {
  {0, "Off"},
  {1, " x2"},
  {2, " x4"},
  {3, " x8"},
  {-1, NULL,}
};

static struct STRTAB nagraudio_bit_list[] = {
  {1, " 8 bits"},
  {0, "16 bits"},
  {-1, NULL,}
};

// For configuration file only
static struct STRTAB nagraudiotab[] = {
  {NAGRAUDIO_AUTO, "auto"},
  {NAGRAUDIO_ON, "yes"},
  {NAGRAUDIO_OFF, "no"},
  {-1, NULL}
};

#ifndef DISABLE_ALL_FILTERS
struct STRTAB bluefilter_list[] = {
  {BLUEFILTER_OFF, "off"},
#ifdef ENABLE_KIZKOOL25
  {BLUEFILTER_K25, "kizkool_2_5"},
#endif
#ifdef ENABLE_KIZKOOL28
  {BLUEFILTER_K28, "kizkool_2_8"},
#endif
#ifdef ENABLE_KIZKOOL30
  {BLUEFILTER_K30, "kizkool_3_0"},
#endif
#ifdef ENABLE_KIZKOOL31
  {BLUEFILTER_K31, "kizkool_3_1"},
#endif
#ifdef ENABLE_KIZKOOL32
  {BLUEFILTER_K32, "kizkool_3_2"},
#endif
#ifdef ENABLE_SNABRAB
  {BLUEFILTER_K32SNABRAB, "kizkool/snabrab"},
#endif
#ifdef ENABLE_FASOUIL
  {BLUEFILTER_FAS, "fasouil"},
#endif
  {-1, NULL}
};
#endif

struct STRTAB black_border_list[] = {
  {BLACK_OFF,  "off"},
  {BLACK_AUTO, "auto"},
  {BLACK_16_9, "16:9"},
  {BLACK_20_9, "20:9"},
  {BLACK_INC,  "+"},
  {BLACK_DEC,  "-"},
  {-1, NULL}
};

struct STRTAB on_off_list[] = {
  {TRUE, "on"},
  {FALSE, "off"},
  {-1, NULL}
};

#ifdef ENABLE_FASOUIL
struct STRTAB superpose_list[] = {
  {SUPERPOSE_MOVING, "moving"},
  {SUPERPOSE_STATIC, "static"},
  {-1, NULL}
};
#endif

static char *help_msg[] = {
  "       -a     turn  off the automode feature. This means that the\n"
  "              default mode will be no descrambling.\n",
  
  "       -w     build a new vck-Data (i.e. overwrite the  old  one)\n"
  "              at xawdecode start up.  For this you must obviously\n"
  "              have a relevant vcr-Data (sendername.vcr).\n",

  "       -1     select NAGRA decryption.\n",
  
  "       -5     select VCRYPT decryption and PAL tuner mode.\n",

  "       -6     select SCRYPT decryption and PAL tuner mode.\n",

  "       -7     select HCRYPT decryption and PAL tuner mode.\n",

  "       -8     select XCRYPT decryption and PAL tuner mode.\n",

  "       -9     select KCRYPT decryption and PAL tuner mode.\n",

  "       -p     select NAGRA decryption and PAL tuner mode.\n",

  "       -l couple\n"
  "              set the number of couples to use for the corelation\n"
  "              used by nagra decryption.\n",
  NULL};

static Widget tv;

/*--- externs ------------------------------------------------------------*/

void set_couples(int i);
void set_nagraudio_auto_mode (int mode);
void set_nagraudio_pre_filter (int val);
void set_nagraudio_post_filter (int val);
void set_nagraudio_gain (int gain);
void set_nagraudio_bit_mode (int mode);
int get_nagraudio_auto_mode (void);
int get_nagraudio_pre_filter (void);
int get_nagraudio_post_filter (void);
int get_nagraudio_gain (void);
int get_nagraudio_bit_mode (void);
void start_nagraudio (void);
void stop_nagraudio (void);
void check_user_directory (void);
extern void (*decode_fullframe)
  (unsigned char *dest,
   unsigned char *src,
   int xsize_dest, int xsize_src, int ysize, int dest_xsize);
extern void (*decode_halfframe)
  (unsigned char *dest,
   unsigned char *src,
   int xsize_dest, int xsize_src, int ysize, int dest_xsize);




// from xawdecode.c
extern int alevt_in_use;
extern char title[256];
extern XtWorkProcId idle_id;
extern struct GRABBER *grabbers[];
extern int grabber;
extern int debug;
extern int maxwidth[];
extern int maxheight[];
extern Atom wm_protocols[2];
extern int cur_capture;

// from ???
//extern FILE *vbifd;
extern int vbifd;
extern int bildzaehler;
extern int decode;
extern int highquality;
//extern int pal;
extern int vc;
extern int use_wss_line;
extern int force_not_use_wss_line;
extern int number_couples;
int nagra_coded = 0;
extern int use_nagraudio;
extern int nagraudio_mode;
extern int ybar;
extern int ybar_auto;
extern int blackborder_tolerance;
extern int hw_scaling;

#ifndef DISABLE_ALL_FILTERS
extern int TypeImage;
#ifndef DISABLE_ALL_KIZKOOL
 extern KIZKOOL_PARAM kizkool_param[NB_KIZKOOL_FILTER];
#endif
#ifdef ENABLE_FASOUIL
 extern FASOUIL_PARAM fasouil_param;
#endif
#ifdef ENABLE_SNABRAB
 extern SNABRAB_PARAM snabrab_param;
#endif
#endif

/*--- public variables ----------------------------------------------------*/
#define MAKE_VERSION(a,b,c) (((a)<<16)&0xFF0000) | (((b)<<8)&0x00FF00) | ((c)&0xFF)

unsigned int version = MAKE_VERSION(MAJOR,MINOR,SUB);

PLUGIN xawdecode = {
  plugin_ui_init,
  plugin_set_idle_grabdisplay,
  plugin_parse_cmdline,
  plugin_create_windows,
  plugin_add_entry,
  plugin_close,
  plugin_set_channel,
  plugin_use_alternate,
  plugin_init_colors,
  plugin_read_config,
  plugin_save_config,
  plugin_do_command,
  plugin_pre_config_init,
  plugin_post_config_init,
  plugin_get_ybar,
  plugin_display_help_message,
  plugin_record_movie,
  plugin_is_nagraudio_used
  //plugin_get_vbifd
};

//int automode = 1;
int auto_decode = DECODER_AUTO;
int validzahl = 1;
int scryptburst = 0;
int fps;
char createvck = 0;
int bluefilter = 0;

// #define VBIDEBUG
#define FPAL     4433            // Pal-Fartrger
#define FTEXT    4470
#ifdef VBIDEBUG
#define BL    27                    // bytes pro line
#else
#define BL      6
#endif

int videotext = 1;
unsigned char vbiline[2048];
unsigned char vbibits[2048 / 4];
unsigned char vbibytes[2048 / 4 / 8];
unsigned int sample_bits[BL * 8];


Widget plugin_shell, plugin_paned;
Widget c_decode, c_sound, c_black, c_couples, c_quality;
Widget nagraudio_shell, nagraudio_paned;

#ifndef DISABLE_ALL_FILTERS
Widget bluefilter_shell, bluefilter_paned;
Widget c_bluefilter;
void gray_widgets(Widget **widget_list, Boolean gray);
void adjust_kizkool_widgets(int filter);

#ifndef DISABLE_ALL_KIZKOOL
 Widget c_ksmudgefilter = NULL;
 Widget s_uthreshold    = NULL;
 Widget s_vthreshold    = NULL;
 Widget s_yuvthreshold  = NULL;
 Widget s_pixthreshold  = NULL;
 Widget l_uthreshold    = NULL;
 Widget l_vthreshold    = NULL;
 Widget l_yuvthreshold  = NULL;
 Widget l_pixthreshold  = NULL;

# ifdef TUNE_SNABRAB
   Widget s_snabrab_first_line = NULL;
   Widget s_snabrab_last_line  = NULL;
   Widget s_snabrab_threshold  = NULL;
   Widget l_snabrab_first_line = NULL;
   Widget l_snabrab_last_line  = NULL;
   Widget l_snabrab_threshold  = NULL;
   Widget *kizkool_widgets[] = {&c_ksmudgefilter,
                                &s_uthreshold, &s_vthreshold, &s_yuvthreshold, &s_pixthreshold,
                                &l_uthreshold, &l_vthreshold, &l_yuvthreshold, &l_pixthreshold,
                                &s_snabrab_first_line, &s_snabrab_last_line, &s_snabrab_threshold,
                                &l_snabrab_first_line, &l_snabrab_last_line, &l_snabrab_threshold,
                                NULL};

# else
 Widget *kizkool_widgets[] = {&c_ksmudgefilter,
                              &s_uthreshold, &s_vthreshold, &s_yuvthreshold, &s_pixthreshold,
                              &l_uthreshold, &l_vthreshold, &l_yuvthreshold, &l_pixthreshold,
                              NULL};
# endif
#endif

#ifdef ENABLE_FASOUIL
 Widget c_fsmudgefilter       = NULL;
 Widget c_averadgefilter      = NULL;
 Widget c_superposetrial      = NULL;
 Widget c_unkonnufilter_phase = NULL;
 Widget c_smudgefilter_phase  = NULL;
 Widget s_uv_threshold        = NULL;
 Widget s_green_coef          = NULL;
 Widget l_uv_threshold        = NULL;
 Widget l_green_coef          = NULL;
 Widget *fasouil_widgets[] = {&c_fsmudgefilter, &c_averadgefilter, &c_superposetrial,
                              &c_unkonnufilter_phase, &c_smudgefilter_phase,
                              &s_uv_threshold, &s_green_coef,
                              &l_uv_threshold, &l_green_coef,
                              NULL};
#endif
#endif

enum {
  CB_PLUGIN_VIDEO_DECODER = 15,
  CB_PLUGIN_SOUND_DECODER,
  CB_PLUGIN_BLACK_BORDER,
  CB_PLUGIN_COUPLES,
  CB_PLUGIN_HIGHQUALITY,

  CB_NAGRAUDIO_AUTO,
  CB_NAGRAUDIO_PREFILTER,
  CB_NAGRAUDIO_POSTFILTER,
  CB_NAGRAUDIO_OUTGAIN,
  CB_NAGRAUDIO_BIT_MODE,

  CB_BLUEFILTER_FILTER,
  CB_BLUEFILTER_KIZKOOL_SMUDGE,
  CB_BLUEFILTER_FASOUIL_SMUDGE,
  CB_BLUEFILTER_FASOUIL_AVERAGE,
  CB_BLUEFILTER_FASOUIL_SUPERPOSE,
  CB_BLUEFILTER_FASOUIL_1KONNU_PHASE,
  CB_BLUEFILTER_FASOUIL_SMUDGE_PHASE
};


/*--- tv -----------------------------------------------------------------*/
char * tv_TranslationTable =
"<Key>5:            CryptModedirekt(VCRYPT)  \n\
 <Key>6:            CryptModedirekt(SCRYPT)  \n\
 <Key>7:            CryptModedirekt(HCRYPT)  \n\
 <Key>8:            CryptModedirekt(XCRYPT)  \n\
 Ctrl<Key>N:        Nagraudio()              \n\
 <Key>N:            CryptModedirekt(NAGRA)   \n\
 <Key>K:            Plugin()                 \n\
 <Key>q:            Quality()                \n\
 <Key>i:            FPS()                    \n\
 Ctrl<Key>S:        SoundMode()              \n\
 <Key>S:            StatusLine()             \n\
 <Key>Insert:       IncCouple()              \n\
 <Key>Delete:       DecCouple()              \n\
 <Key>BackSpace:    CryptMode()              \n\
 <Key>Home:         SetBlack(dec)            \n\
 <Key>End:          SetBlack(inc)            \n\
 <Key>D:            Decode()                 \n\ "
#ifndef DISABLE_ALL_FILTERS
"Alt<Key>B:         BlueFilterToggle()       \n\
 Ctrl<Key>B:        BlueFilter()             \n\ "
#endif
"<Key>B:            SetBurst()               \n\ "
#if (!defined(DISABLE_ALL_FILTERS))&&(!defined(DISABLE_ALL_KIZKOOL))
 "Alt<Key>X:         SmudgeFilterToggle()     \n\
  Alt<Key>U:         FilterThreshold(U,dec)   \n\
  Ctrl<Key>U:        FilterThreshold(U,inc)   \n\
  Alt<Key>V:         FilterThreshold(V,dec)   \n\
  Ctrl<Key>V:        FilterThreshold(V,inc)   \n\
  Alt<Key>Y:         FilterThreshold(YUV,dec) \n\
  Ctrl<Key>Y:        FilterThreshold(YUV,inc) \n\
  Alt<Key>P:         FilterThreshold(PIX,dec) \n\
  Ctrl<Key>P:        FilterThreshold(PIX,inc) \n\
  <Key>P:            SmudgeFilterPhaseToggle()\n\
  Alt<Key>K:         FilterThreshold(SNAB_F,dec) \n\
  Ctrl<Key>K:        FilterThreshold(SNAB_F,inc) \n\
  Alt<Key>L:         FilterThreshold(SNAB_L,dec) \n\
  Ctrl<Key>L:        FilterThreshold(SNAB_L,inc) \n\
  Alt<Key>semicolon: FilterThreshold(SNAB_T,dec) \n\
  Ctrl<Key>semicolon: FilterThreshold(SNAB_T,inc)"
#endif
;


static Boolean
idle_grabdisplay2 (XtPointer data)
{
  static long count, lastsec;
  struct timeval tv;
  struct timezone tz;

  int i, j, l, n;
  int soffset;
  static int text_lines_found = 0;

  if (decode==NAGRA)
    {
      switch (auto_decode)
        {
        case  DECODER_ON:
          nagra_coded = 1;
          break;

        case DECODER_OFF:
          nagra_coded = 0;
          break;

        case DECODER_AUTO:
          for (i = 0; i < BL * 8; i++)
            sample_bits[i] = FPAL * 8 * i / FTEXT;
          soffset = (sample_bits[1] - sample_bits[0]) / 2 + 1;

          if (alevt_in_use == 0)
            {
              text_lines_found = 0;
              for (l = 0; l < 15; l++)
                {
                  //n = fread (vbiline, 1, 2048, vbifd);
                  n = read(vbifd, vbiline, 2048);

                  // Abtastwerte  triggern
                  for (i = 0; i < 2048; i += 1)
                    {
                      if (vbiline[i] < 95)
                        vbiline[i] = 0;
                      else
                        vbiline[i] = 1;
                    }
                  
                  // Abtastwert -> Bits
                  i = 0;
                  while (vbiline[i] != 1)
                    if (i++ > 2048)
                      break;
                  if (i > 200)
                    continue;
                  for (j = 0; j < BL * 8; j++)
                    vbibits[j] = vbiline[sample_bits[j] + soffset + i];

                  // Bits -> Bytes
                  for (i = 0; i < BL * 8; i += 8)
                    {
                      vbibytes[i / 8] = 0;
                      for (j = 0; j < 8; j++)
                        vbibytes[i / 8] |= vbibits[i + j] << j;    // inverse order
#ifdef VBIDEBUG
                      // if ((vbibytes[0]==0x55) && (vbibytes[4]==0x15)) 
                      if (vbibytes[0] == 0x55)
                        printf ("%02x,", vbibytes[i / 8]);
#endif
                    }

#ifdef VBIDEBUG
                  // if ((vbibytes[0]==0x55) && (vbibytes[4]==0x15)) 
                  if (vbibytes[0] == 0x55)
                    printf ("\n");
#endif

                  if (vbibytes[0] == 0x55)
                    text_lines_found = 1;
              
                  if ((vbibytes[0] == 0x55) && (vbibytes[4] == 0x15))
                    {
                      nagra_coded = vbibytes[5] & 0x20;
                    }
                }
              if (!text_lines_found)
                nagra_coded = 0;

            }
          break;
        }

    } // ENDE if decode==nagra



  gettimeofday (&tv, &tz);
  if (tv.tv_sec != lastsec)
    {
      if (lastsec == tv.tv_sec - 1)
        fps = count;
      //      printf ("fps:%i\n",fps); fflush(stdout);
      lastsec = tv.tv_sec;
      count = 0;
    }
  count++;

  if ((NULL == grabbers[grabber]->grab_scr) ||
      (-1 == video_displayframe (grabbers[grabber]->grab_scr)))
    {
      idle_id = 0;
      usleep (300000);
      // return TRUE;
    }
  return FALSE;
}



void
set_video_decoder (int mode)
{
  char *label = (char *) NULL;

  decode = mode;

  switch (mode)
    {
    case CLEAR:
      label = "CLEAR";
      break;
    case NAGRA:
      label = "Nagra";
      break;
    case VCRYPT:
      label = "VCRYPT";
      break;
    case SCRYPT:
      label = "SCRYPT";
      break;
    case HCRYPT:
      label = "HCRYPT";
      break;
    case XCRYPT:
      label = "XCRYPT";
      break;
    case KCRYPT:
      label = "KCRYPT";
      break;
    }

  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Video Decoder", label);
  if (c_decode)
    XtVaSetValues (c_decode, XtNlabel, title, NULL);

  sprintf (title, "Cryptmode: %s", label);
  set_timer_title ();
}

void
set_sound_decoder (int mode, char *label)
{
  //  char *label = (char *) NULL;

  switch (mode)
    {
    case CLEAR:
      if (use_nagraudio)
        {
          stop_nagraudio ();
          mixer_unmute ();
          mixer_uncapture ();
          use_nagraudio = 0;
        }
      if (label == NULL)
        label = "CLEAR";
      break;

    case NAGRA:
      if (!use_nagraudio)
        {
          mixer_mute ();
          mixer_capture ();
          start_nagraudio ();
          use_nagraudio = 1;
        }
      if (label == NULL)
        label = "Nagra";
      break;

    case AUTO:
      // There's nothing to do except updating the Option window entry
      if (label == NULL)
        label = "Auto";
      break;
    }

  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Sound Decoder", label);
  if (c_sound)
    XtVaSetValues (c_sound, XtNlabel, title, NULL);
}


void
set_black_border (int mode)
{
  char *label = (char *) NULL;

  if (cur_capture != CAPTURE_GRABDISPLAY)
    return;

  switch (mode)
    {
    case BLACK_OFF:
      ybar_auto = FALSE;
      ybar = 1;
      label = "OFF";
      break;

    case BLACK_AUTO:
      ybar_auto = TRUE;
      ybar = 1;
      label = "Auto";
      break;

    case BLACK_16_9:
      ybar_auto = FALSE;
      ybar = (maxheight[cur_norm]-maxwidth[cur_norm]*9/16)/2;
      label = "16:9";
      break;

    case BLACK_20_9:
      ybar_auto = FALSE;
      ybar = (maxheight[cur_norm]-maxwidth[cur_norm]*9/20)/2;
      label = "20:9";
      break;

    case BLACK_DEC:
      ybar_auto = FALSE;
      if (ybar > 1)
        ybar--;
      break;

    case BLACK_INC:
      ybar_auto = FALSE;
      if (ybar < /*90*/ 180)
        ybar++;
      break; 
    }

if (mode < BLACK_DEC)
  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Black Border", label);
 else
  sprintf (title, "%-" LABEL_WIDTH "s: %d", "Black Border", ybar);
 
  if (c_black)
    XtVaSetValues (c_black, XtNlabel, title, NULL);
}

#ifndef DISABLE_ALL_FILTERS
void
set_bluefilter(int filter)
{
  if (debug)
    fprintf(stderr, "set_bluefilter: filter = %d\n", filter);

  bluefilter = filter;

  // Update Filter Window
  if (c_bluefilter)
    {
      sprintf (title, "%-" LABEL_WIDTH "s: %s", "Blue Filter",
               int_to_str (bluefilter, bluefilter_list));
      XtVaSetValues (c_bluefilter, XtNlabel, title, NULL);
    }

  switch(bluefilter)
    {
    case BLUEFILTER_OFF:
      sprintf (title, "Blue Filter: OFF");
      set_timer_title ();
#ifndef DISABLE_ALL_KIZKOOL
      gray_widgets(kizkool_widgets, False);
#endif
#ifdef ENABLE_FASOUIL
      gray_widgets(fasouil_widgets, False);
#endif
      // Do nothing
      break;

#ifndef DISABLE_ALL_KIZKOOL

#ifdef ENABLE_KIZKOOL25
    case BLUEFILTER_K25:
#endif
#ifdef ENABLE_KIZKOOL28
    case BLUEFILTER_K28:
#endif
#ifdef ENABLE_KIZKOOL30
    case BLUEFILTER_K30:
#endif
#ifdef ENABLE_KIZKOOL31
    case BLUEFILTER_K31:
#endif
#ifdef ENABLE_KIZKOOL32
    case BLUEFILTER_K32:
#endif
#ifdef ENABLE_SNABRAB
    case BLUEFILTER_K32SNABRAB:
#endif
#ifdef ENABLE_FASOUIL
      fasouil_param.bFiltreViolet = FALSE;
      gray_widgets(fasouil_widgets, False);
#endif

      gray_widgets(kizkool_widgets, True);

      // Adjust some filter pecularities
      adjust_kizkool_widgets(bluefilter);
      
      // Update Filter Window
      if (c_ksmudgefilter)
        {
          sprintf (title, "%-" LABEL_WIDTH "s: %s", "Smudge Filter",
                   int_to_str ( kizkool_param[bluefilter - 1].smudge, on_off_list));
          XtVaSetValues (c_ksmudgefilter, XtNlabel, title, NULL);
        }
      
      if (s_uthreshold)
        set_float (s_uthreshold, XtNtopOfThumb,
                   (float) kizkool_param[bluefilter - 1].iUFilterThresholdPCent / 100);

      if (s_vthreshold)
        set_float (s_vthreshold, XtNtopOfThumb,
                   (float) kizkool_param[bluefilter - 1].iVFilterThresholdPCent / 100);

      if (s_yuvthreshold)
        set_float (s_yuvthreshold, XtNtopOfThumb,
                   (float) kizkool_param[bluefilter - 1].iYUVFilterThresholdPCent / 100);

      if (s_pixthreshold)
        set_float (s_pixthreshold, XtNtopOfThumb,
                   (float) kizkool_param[bluefilter - 1].iPixFilterThreshold / 100);

#if defined(TUNE_SNABRAB) && defined(ENABLE_SNABRAB)
      if (s_snabrab_first_line)
        set_float (s_snabrab_first_line, XtNtopOfThumb,
                   (float) snabrab_param.FirstLine / 288);

      if (s_snabrab_last_line)
        set_float (s_snabrab_last_line, XtNtopOfThumb,
                   (float) snabrab_param.LastLine / 288);

      if (s_snabrab_threshold)
        set_float (s_snabrab_threshold, XtNtopOfThumb,
                   (float) snabrab_param.Threshold / 288);
#endif

      sprintf (title, "Blue Filter: %s", int_to_str (bluefilter, bluefilter_list) );
      set_timer_title ();
      break;
#endif // DISABLE_ALL_KIZKOOL

#ifdef ENABLE_FASOUIL
    case BLUEFILTER_FAS:
      fasouil_param.bFiltreViolet = TRUE;
      gray_widgets(fasouil_widgets, True);
#ifndef DISABE_ALL_KIZKOOL
      gray_widgets(kizkool_widgets, False);
#endif

      // Update Filter Window
      if (c_fsmudgefilter)
        {
          sprintf (title, "%-" LABEL_WIDTH "s: %s", "Smudge Filter",
                   int_to_str (fasouil_param.bFiltreAntiBavure, on_off_list));
          XtVaSetValues (c_fsmudgefilter, XtNlabel, title, NULL);
        }

      if (c_averadgefilter)
        {
          sprintf (title, "%-" LABEL_WIDTH "s: %s",
                   "Average Filter", int_to_str (fasouil_param.bFiltreMoyenneur, on_off_list));
          XtVaSetValues (c_averadgefilter, XtNlabel, title, NULL);
        }

      if (c_superposetrial)
        {
          sprintf (title, "%-" LABEL_WIDTH "s: %s",
                   "Superpose Trial", int_to_str (fasouil_param.EssaiSuperposition, superpose_list));
          XtVaSetValues (c_superposetrial, XtNlabel, title, NULL);
        }

      if (c_unkonnufilter_phase)
        {
          sprintf (title, "%-" LABEL_WIDTH "s: %s",
                   "1Konnu Phase", int_to_str (fasouil_param.bPhaseFiltreUnKonnus, on_off_list));
          XtVaSetValues (c_unkonnufilter_phase, XtNlabel, title, NULL);
        }

      if (c_smudgefilter_phase)
        {
          sprintf (title, "%-" LABEL_WIDTH "s: %s", "Smudge Phase",
                   int_to_str (fasouil_param.bPhaseFiltreAntiBavure, on_off_list));
          XtVaSetValues (c_smudgefilter_phase, XtNlabel, title, NULL);
        }

      if (s_uv_threshold)
        set_float (s_uv_threshold, XtNtopOfThumb,
                   (float) fasouil_param.iUVThresholdPCent / 100);

      if (s_green_coef)
        set_float (s_green_coef, XtNtopOfThumb,
                   (float) fasouil_param.CoefFiltreVert / 100);

      sprintf (title, "Blue Filter: %s", int_to_str (bluefilter, bluefilter_list) );
      set_timer_title ();
      break;
#endif // ENABLE_FASOUIL

    default:
      // Do nothing
      break;
    }

}

void
set_bluefilter_params (int filter, ...)
     /*
 int blue, int smudge, int average, int superpose,
 int unkonnu_phase, int smudge_phase,
 int uv_threshold, int green_coef
     */
{
  va_list argp;
  int arg;
  int val;
  int nbarg = 1;
  
  // read all the *COUPLES* of parameters
  va_start(argp, filter);
  while ((arg = va_arg(argp, int)) != 0)
    {
      val = va_arg(argp, int);
      nbarg++;

      if (debug)
        fprintf(stderr, "set_bluefilter_params: filter = %d, arg = %d, val =%d\n", filter, arg, val);

      switch(filter)
        {
        case BLUEFILTER_OFF:
          // Do nothing
          break;

#ifndef DISABLE_ALL_KIZKOOL
#ifdef ENABLE_KIZKOOL25
    case BLUEFILTER_K25:
#endif
#ifdef ENABLE_KIZKOOL28
    case BLUEFILTER_K28:
#endif
#ifdef ENABLE_KIZKOOL30
    case BLUEFILTER_K30:
#endif
#ifdef ENABLE_KIZKOOL31
    case BLUEFILTER_K31:
#endif
#ifdef ENABLE_KIZKOOL32
    case BLUEFILTER_K32:
#endif
#ifdef ENABLE_SNABRAB
    case BLUEFILTER_K32SNABRAB:
#endif
          switch(arg)
            {
            case KIZKOOL_SMUDGE:
              // Set parameter
              kizkool_param[filter - 1].smudge = val;

              // Update Filter Window
              if (c_ksmudgefilter)
                {
                  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Smudge Filter",
                           int_to_str (kizkool_param[filter - 1].smudge , on_off_list));
                  XtVaSetValues (c_ksmudgefilter, XtNlabel, title, NULL);
                }
              break;

            case KIZKOOL_UTHRESHOLD:
              // Set parameter
              kizkool_param[filter - 1].iUFilterThresholdPCent =
                (int) rint ((float) (val * 100) / 65535.0);
              kizkool_param[filter - 1].nUThreshold =
                (int) (kizkool_param[filter - 1].iUFilterThresholdPCent * 0.01 *
                       0xff);

              // Update Filter Window
              if (s_uthreshold)
                set_float (s_uthreshold, XtNtopOfThumb,
                           (float) kizkool_param[bluefilter - 1].iUFilterThresholdPCent / 100);

              sprintf (title, "U Filter Threshold: %d %%",
                       kizkool_param[bluefilter - 1].iUFilterThresholdPCent);
              set_timer_title();
              break;

            case KIZKOOL_VTHRESHOLD:
              // Set parameter
              kizkool_param[filter - 1].iVFilterThresholdPCent =
                (int) rint ((float) (val * 100) / 65535.0);
              kizkool_param[filter - 1].nVThreshold =
                (int) (kizkool_param[filter - 1].iVFilterThresholdPCent * 0.01 *
                       0xff);

              // Update Filter Window
              if (s_vthreshold)
                set_float (s_vthreshold, XtNtopOfThumb,
                           (float) kizkool_param[bluefilter - 1].iVFilterThresholdPCent / 100);

              sprintf (title, "V Filter Threshold: %d %%",
                       kizkool_param[bluefilter - 1].iVFilterThresholdPCent);
              set_timer_title();
              break;

            case KIZKOOL_YUVTHRESHOLD:
              // Set parameter
              kizkool_param[filter - 1].iYUVFilterThresholdPCent =
                (int) rint ((float) (val * 100) / 65535.0);
              kizkool_param[filter - 1].nYUVThreshold =
                (int) (kizkool_param[filter - 1].iYUVFilterThresholdPCent * 0.01 *
                       0xff);

              // Update Filter Window
              if (s_yuvthreshold)
                set_float (s_yuvthreshold, XtNtopOfThumb,
                           (float) kizkool_param[bluefilter - 1].iYUVFilterThresholdPCent / 100);

              sprintf (title, "YUV Filter Threshold: %d %%",
                       kizkool_param[bluefilter - 1].iYUVFilterThresholdPCent);
              set_timer_title();
              break;

            case KIZKOOL_PIXFILTERTHRESHOLD:
              // Set parameter
              kizkool_param[filter - 1].iPixFilterThreshold =
                (int) rint ((float) (val * 100) / 65535.0);

              // Update Filter Window
              if (s_pixthreshold)
                set_float (s_pixthreshold, XtNtopOfThumb,
                           (float) kizkool_param[bluefilter - 1].iPixFilterThreshold / 100);

              sprintf (title, "Pix Filter Threshold: %d %%",
                       kizkool_param[bluefilter - 1].iPixFilterThreshold);
              set_timer_title();
              break;

#ifdef ENABLE_SNABRAB
            case KIZKOOL_SNABRAB_FIRSTLINE:
              // Set parameter
              snabrab_param.FirstLine =
                (int) rint ((float) (val * 288) / 65535.0);

#ifdef TUNE_SNABRAB
              // Update Filter Window
              if (s_snabrab_first_line)
                set_float (s_snabrab_first_line, XtNtopOfThumb,
                           (float) snabrab_param.FirstLine / 288);
#endif // TUNE_SNABRAB
              sprintf (title, "Snabrab First Line: %d",
                       snabrab_param.FirstLine);
              set_timer_title();
              break;

            case KIZKOOL_SNABRAB_LASTLINE:
              // Set parameter
              snabrab_param.LastLine =
                (int) rint ((float) (val * 288) / 65535.0);

#ifdef TUNE_SNABRAB
              // Update Filter Window
              if (s_snabrab_last_line)
                set_float (s_snabrab_last_line, XtNtopOfThumb,
                           (float) snabrab_param.LastLine / 288);
#endif // TUNE_SNABRAB

              sprintf (title, "Snabrab Last Line: %d",
                       snabrab_param.LastLine);
              set_timer_title();
              break;

            case KIZKOOL_SNABRAB_THRESHOLD:
              // Set parameter
              snabrab_param.Threshold =
                (int) rint ((float) (val * 288) / 65535.0);

#ifdef TUNE_SNABRAB
              // Update Filter Window
              if (s_snabrab_threshold)
                set_float (s_snabrab_threshold, XtNtopOfThumb,
                           (float) snabrab_param.Threshold / 288);
#endif // TUNE_SNABRAB

              sprintf (title, "Snabrab Threshold: %d",
                       snabrab_param.Threshold);
              set_timer_title();
              break;
#endif // ENABLE_SNABRAB

            }
          break;
#endif // DISABLE_ALL_KIZKOOL

#ifdef ENABLE_FASOUIL
        case BLUEFILTER_FAS:
          switch(arg)
            {
            case FASOUIL_SMUDGE:
              // Set parameter
              fasouil_param.bFiltreAntiBavure = val;

              // Update Filter Window
              if (c_fsmudgefilter)
                {
                  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Smudge Filter",
                           int_to_str (fasouil_param.bFiltreAntiBavure, on_off_list));
                  XtVaSetValues (c_fsmudgefilter, XtNlabel, title, NULL);
                }
              break;
  
            case FASOUIL_AVERAGE:
              // Set parameter
              fasouil_param.bFiltreMoyenneur = val;

              // Update Filter Window
              if (c_averadgefilter)
                {
                  sprintf (title, "%-" LABEL_WIDTH "s: %s",
                           "Average Filter", int_to_str (fasouil_param.bFiltreMoyenneur, on_off_list));
                  XtVaSetValues (c_averadgefilter, XtNlabel, title, NULL);
                }
              break;

            case FASOUIL_SUPERPOSE:
              // Set parameter
              fasouil_param.EssaiSuperposition = val;

              // Update Filter Window
              if (c_superposetrial)
                {
                  sprintf (title, "%-" LABEL_WIDTH "s: %s",
                           "Superpose Trial", int_to_str (fasouil_param.EssaiSuperposition, superpose_list));
                  XtVaSetValues (c_superposetrial, XtNlabel, title, NULL);
                }
              break;

            case FASOUIL_UNKONNU_PHASE:
              // Set parameter
              fasouil_param.bPhaseFiltreUnKonnus = val;

              // Update Filter Window
              if (c_unkonnufilter_phase)
                {
                  sprintf (title, "%-" LABEL_WIDTH "s: %s",
                           "1Konnu Phase", int_to_str (fasouil_param.bPhaseFiltreUnKonnus, on_off_list));
                  XtVaSetValues (c_unkonnufilter_phase, XtNlabel, title, NULL);
                }
              break;

            case FASOUIL_SMUDGE_PHASE:
              // Set parameter
              fasouil_param.bPhaseFiltreAntiBavure = val;

              // Update Filter Window
              if (c_smudgefilter_phase)
                {
                  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Smudge Phase",
                           int_to_str (fasouil_param.bPhaseFiltreAntiBavure, on_off_list));
                  XtVaSetValues (c_smudgefilter_phase, XtNlabel, title, NULL);
                }
              break;

            case FASOUIL_UV_THRESHOLD:
              // Set parameter
              fasouil_param.iUVThresholdPCent =
                (int) rint ((float) (val * 100) / 65535.0);

              // Update Filter Window
              if (s_uv_threshold)
                set_float (s_uv_threshold, XtNtopOfThumb,
                           (float) fasouil_param.iUVThresholdPCent / 100);

              sprintf (title, "UV Filter Threshold: %d %%",
                       fasouil_param.iUVThresholdPCent);
              set_timer_title ();
              break;

            case FASOUIL_GREEN_COEFF:
              // Set parameter
              fasouil_param.CoefFiltreVert =
                (int) rint ((float) (val * 255) / 65535.0);

              // Update Filter Window
              if (s_green_coef)
                set_float (s_green_coef, XtNtopOfThumb,
                           (float) fasouil_param.CoefFiltreVert / 255);

              sprintf (title, "Green Filter Threshold: %d",
                       fasouil_param.CoefFiltreVert);
              set_timer_title ();
              break;

            }
          break;
#endif // ENABLE_FASOUIL

        default:
          // Do nothing
          break;
        }
    }
  va_end(argp);


}
#endif // DISABLE_ALL_FILTERS


/*--- plugin windows ------------------------------------------------------*/

struct CALL_ACTION call_plugin = { 0, "Plugin", {NULL} };
struct CALL_ACTION call_nagraudio = { 0, "Nagraudio", {NULL} };
struct CALL_ACTION call_bluefilter = { 0, "BlueFilter", {NULL} };

void
plugin_menu_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  int cd = (int) clientdata;
  int j;

  switch (cd)
    {
      /* Plugin window entries */

    case CB_PLUGIN_VIDEO_DECODER:
      if (-1 != (j = popup_menu (widget, "Video Decoder", decode_list)))
        {
          set_video_decoder (j);
        }
      break;

    case CB_PLUGIN_SOUND_DECODER:
      if (-1 != (j = popup_menu (widget, "Sound Decoder", sound_list)))
        {
          nagraudio_mode = j;
          if (nagraudio_mode == NAGRAUDIO_ON)
            set_sound_decoder (NAGRA, NULL);
          else if (nagraudio_mode == NAGRAUDIO_OFF)
            set_sound_decoder (CLEAR, NULL);
          else if (nagraudio_mode == NAGRAUDIO_AUTO)
            set_sound_decoder (AUTO, NULL);
        }
      break;

    case CB_PLUGIN_BLACK_BORDER:
      if (-1 != (j = popup_menu (widget, "Black Border", black_border_list)))
        {
          set_black_border(j);
        }
      break;

    case CB_PLUGIN_COUPLES:
      if (-1 != (j = popup_menu (widget, "Couples", couples_list)))
        {
           set_couples(j);
           sprintf (title, "%-" LABEL_WIDTH "s: %s", "Couples",
                    int_to_str (j, couples_list));
           XtVaSetValues (widget, XtNlabel, title, NULL);
        }
      break;

    case CB_PLUGIN_HIGHQUALITY:
      if (-1 != (j = popup_menu (widget, "HighQuality", on_off_list)))
        {
          highquality = j;
          sprintf (title, "%-" LABEL_WIDTH "s: %s", "HighQuality",
                   int_to_str (highquality, on_off_list));
          set_timer_title ();
          XtVaSetValues (c_quality, XtNlabel, title, NULL);
        }
      break;

      /* Nagraudio window entries */
    case CB_NAGRAUDIO_AUTO:
      if (-1 !=
          (j = popup_menu (widget, "Nagraudio Auto", nagraudio_auto_list)))
        {
          set_nagraudio_auto_mode (j);
          sprintf (title, "%-" LABEL_WIDTH "s: %s", "Automatic Mode",
                   int_to_str (j, nagraudio_auto_list));
          XtVaSetValues (widget, XtNlabel, title, NULL);
        }
      break;

    case CB_NAGRAUDIO_PREFILTER:
      if (-1 !=
          (j =
           popup_menu (widget, "Nagraudio PreFilter", nagraudio_filter_list)))
        {
          set_nagraudio_pre_filter (j);
          sprintf (title, "%-" LABEL_WIDTH "s: %s", "Prefilter",
                   int_to_str (j, nagraudio_filter_list));
          XtVaSetValues (widget, XtNlabel, title, NULL);
        }
      break;

    case CB_NAGRAUDIO_POSTFILTER:
      if (-1 !=
          (j =
           popup_menu (widget, "Nagraudio PostFilter",
                       nagraudio_filter_list)))
        {
          set_nagraudio_post_filter (j);
          sprintf (title, "%-" LABEL_WIDTH "s: %s", "Postfilter",
                   int_to_str (j, nagraudio_filter_list));
          XtVaSetValues (widget, XtNlabel, title, NULL);
        }
      break;

    case CB_NAGRAUDIO_OUTGAIN:
      if (-1 !=
          (j =
           popup_menu (widget, "Nagraudio Out Gain", nagraudio_gain_list)))
        {
          set_nagraudio_gain (j);
          sprintf (title, "%-" LABEL_WIDTH "s: %s", "Output Gain",
                   int_to_str (j, nagraudio_gain_list));
          XtVaSetValues (widget, XtNlabel, title, NULL);
        }
      break;

    case CB_NAGRAUDIO_BIT_MODE:
      if (-1 !=
          (j = popup_menu (widget, "Nagraudio Bit Mode", nagraudio_bit_list)))
        {
          set_nagraudio_bit_mode (j);
          sprintf (title, "%-" LABEL_WIDTH "s: %s", "Bit Mode",
                   int_to_str (j, nagraudio_bit_list));
          XtVaSetValues (widget, XtNlabel, title, NULL);
        }
      break;

      /* BlueFilter window entries */
#ifndef DISABLE_ALL_FILTERS
    case CB_BLUEFILTER_FILTER:
      if (-1 != (j = popup_menu (widget, "Blue Filter", bluefilter_list)))
        {
          set_bluefilter (j);
        }
      break;

#ifndef DISABLE_ALL_KIZKOOL
    case CB_BLUEFILTER_KIZKOOL_SMUDGE:
      if (-1 !=
          (j = popup_menu (widget, "Smudge Filter (KizKoool)", on_off_list)))
        {
              if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
                set_bluefilter_params (bluefilter, KIZKOOL_SMUDGE, j, 0);
        }
      break;
#endif // DISABLE_ALL_KIZKOOL

#ifdef ENABLE_FASOUIL
    case CB_BLUEFILTER_FASOUIL_SMUDGE:
      if (-1 !=
          (j = popup_menu (widget, "Smudge Filter (Fasouil)", on_off_list)))
        {
          if (bluefilter == BLUEFILTER_FAS)
            set_bluefilter_params (bluefilter, FASOUIL_SMUDGE, j, 0);
        }
      break;

    case CB_BLUEFILTER_FASOUIL_AVERAGE:
      if (-1 !=
          (j = popup_menu (widget, "Average Filter (Fasouil)", on_off_list)))
        {
          if (bluefilter == BLUEFILTER_FAS)
            set_bluefilter_params (bluefilter, FASOUIL_AVERAGE, j, 0);
        }
      break;

    case CB_BLUEFILTER_FASOUIL_SUPERPOSE:
      if (-1 !=
          (j =
           popup_menu (widget, "Superpose Trial (Fasouil)", superpose_list)))
        {
          if (bluefilter == BLUEFILTER_FAS)
            set_bluefilter_params (bluefilter, FASOUIL_SUPERPOSE, j, 0);
        }
      break;

    case CB_BLUEFILTER_FASOUIL_1KONNU_PHASE:
      if (-1 !=
          (j =
           popup_menu (widget, "1Konnu Filter Phase (Fasouil)", on_off_list)))
        {
          if (bluefilter == BLUEFILTER_FAS)
            set_bluefilter_params (bluefilter, FASOUIL_UNKONNU_PHASE, j, 0);
        }
      break;

    case CB_BLUEFILTER_FASOUIL_SMUDGE_PHASE:
      if (-1 !=
          (j =
           popup_menu (widget, "Smudge Filter Phase (Fasouil)", on_off_list)))
        {
          if (bluefilter == BLUEFILTER_FAS)
            set_bluefilter_params (bluefilter, FASOUIL_SMUDGE_PHASE, j, 0);
        }
      break;
#endif // ENABLE_FASOUIL

#endif // DISABLE_ALL_FILTERS

    default:
      break;
    }

  if (title[0] != 0)
    set_timer_title ();
}

void
plugin_jump_scb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  char *name;
  int value;

  name = XtName (XtParent (widget));
  value = (int) (*(float *) call_data * 65535);
#if 0
  fprintf (stderr, "jump to %f (%s/%d)\n", *(float *) call_data, name, value);
#endif

#ifndef DISABLE_ALL_FILTERS

#ifndef DISABLE_ALL_KIZKOOL

  if (0 == strcmp ("uthreshold", name))
    {
      if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
        set_bluefilter_params (bluefilter, KIZKOOL_UTHRESHOLD, value, 0);
    }

  else if (0 == strcmp ("vthreshold", name))
    {
      if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
        set_bluefilter_params (bluefilter, KIZKOOL_VTHRESHOLD, value, 0);
    }

  else if (0 == strcmp ("yuvthreshold", name))
    {
      if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
        set_bluefilter_params (bluefilter, KIZKOOL_YUVTHRESHOLD, value, 0);
    }

  else if (0 == strcmp ("pixthreshold", name))
    {
      if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
        set_bluefilter_params (bluefilter, KIZKOOL_PIXFILTERTHRESHOLD, value, 0);
    }

#if defined(ENABLE_SNABRAB) &&  defined(TUNE_SNABRAB)
  else if (0 == strcmp ("snabrab_first_line", name))
    {
      if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
        set_bluefilter_params (bluefilter, KIZKOOL_SNABRAB_FIRSTLINE, value, 0);
    }
  else if (0 == strcmp ("snabrab_last_line", name))
    {
      if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
        set_bluefilter_params (bluefilter, KIZKOOL_SNABRAB_LASTLINE, value, 0);
    }
  else if (0 == strcmp ("snabrab_threshold", name))
    {
      if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
        set_bluefilter_params (bluefilter, KIZKOOL_SNABRAB_THRESHOLD, value, 0);
    }
#endif // defined(ENABLE_SNABRAB) &&  defined(TUNE_SNABRAB)

#endif // DISABLE_ALL_KIZKOOL

#ifdef ENABLE_FASOUIL

#ifndef DISABLE_ALL_KIZKOOL
  else if (0 == strcmp ("uv_threshold", name))
#else
  if (0 == strcmp ("uv_threshold", name))
#endif
    {
      if (bluefilter == BLUEFILTER_FAS)
        set_bluefilter_params (bluefilter, FASOUIL_UV_THRESHOLD, value, 0);
    }

  else if (0 == strcmp ("green_coef", name))
    {
      if (bluefilter == BLUEFILTER_FAS)
        set_bluefilter_params (bluefilter, FASOUIL_GREEN_COEFF, value, 0);
    }
#endif

#endif // DISABLE_ALL_FILTERS
}

void
plugin_scroll_scb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  int move = (int) call_data;
  Dimension length;
  float shown, top1, top2;

  XtVaGetValues (widget,
                 XtNlength, &length,
                 XtNshown, &shown, XtNtopOfThumb, &top1, NULL);

  top2 = top1 + (float) move / length / 5;
  if (top2 < 0)
    top2 = 0;
  if (top2 > 1)
    top2 = 1;

#if 0
  fprintf (stderr, "scroll by %d\tlength %d\tshown %f\ttop %f => %f\n",
           move, length, shown, top1, top2);
#endif

  plugin_jump_scb (widget, clientdata, &top2);
}

void
create_pluginwin(Widget shell)
{

  Widget c;
  XrmDatabase rb;

  rb = XtDatabase(XtDisplay(shell));
  XrmPutStringResource(&rb, "Plugin.paned.internalBorderWidth", "0");
  XrmPutStringResource(&rb, "Plugin*decode.label",         "Video Decoder      >");
  XrmPutStringResource(&rb, "Plugin*sound.label",          "Sound Decoder      >       Ctrl+S");
  XrmPutStringResource(&rb, "Plugin*black.label",          "Black Border    : OFF");
  XrmPutStringResource(&rb, "Plugin*nagraudiowin.label",   "Nagraudio Settings         Ctrl+N");
  XrmPutStringResource(&rb, "Plugin*bluefilterwin.label",  "Blue Filter Settings       Ctrl+B");
  XrmPutStringResource(&rb, "Plugin*highquality.label",    "HighQuality     : OFF");
  XrmPutStringResource(&rb, "Plugin*Label.shadowWidth",    "0");
  XrmPutStringResource(&rb, "Plugin*Command.shadowWidth",  "0");
  XrmPutStringResource(&rb, "Plugin*Command.translations", "#override\n<EnterWindow>: SetShadow(1)\n<LeaveWindow>: SetShadow(0)");



  plugin_shell = XtVaAppCreateShell ("Plugin", "Xawdecode",
                                        topLevelShellWidgetClass,
                                        XtDisplay(shell),
                                        XtNclientLeader, shell, NULL);

  XtOverrideTranslations (plugin_shell, XtParseTranslationTable
                          ("<Message>WM_PROTOCOLS: Plugin()"));

  plugin_paned =
    XtVaCreateManagedWidget ("paned", panedWidgetClass, plugin_shell,
                             NULL);

  XtOverrideTranslations (plugin_paned, XtParseTranslationTable
                          ("<Key>Escape: Plugin()"));

  // hier
  c_decode = XtVaCreateManagedWidget ("decode", commandWidgetClass, plugin_paned,
                                      PANED_FIX, NULL);
  XtAddCallback (c_decode, XtNcallback, plugin_menu_cb, (XtPointer) CB_PLUGIN_VIDEO_DECODER);


  c_sound = XtVaCreateManagedWidget ("sound", commandWidgetClass, plugin_paned,
                                     PANED_FIX, NULL);
  XtAddCallback (c_sound, XtNcallback, plugin_menu_cb, (XtPointer) CB_PLUGIN_SOUND_DECODER);


  c_black = XtVaCreateManagedWidget ("black", commandWidgetClass, plugin_paned,
                                     PANED_FIX, NULL);
  XtAddCallback (c_black, XtNcallback, plugin_menu_cb, (XtPointer) CB_PLUGIN_BLACK_BORDER);


  c = XtVaCreateManagedWidget ("nagraudiowin", commandWidgetClass, plugin_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, action_cb, (XtPointer) & call_nagraudio);

#ifndef DISABLE_ALL_FILTERS
  c = XtVaCreateManagedWidget ("bluefilterwin", commandWidgetClass, plugin_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, action_cb, (XtPointer) & call_bluefilter);
#endif

  c_couples = XtVaCreateManagedWidget ("couples", commandWidgetClass, plugin_paned,
                                      PANED_FIX, NULL);
  XtAddCallback (c_couples, XtNcallback, plugin_menu_cb, (XtPointer) CB_PLUGIN_COUPLES);

  c_quality = XtVaCreateManagedWidget ("quality", commandWidgetClass, plugin_paned,
                                      PANED_FIX, NULL);
  XtAddCallback (c_quality, XtNcallback, plugin_menu_cb, (XtPointer) CB_PLUGIN_HIGHQUALITY);

}

void
create_nagraudiowin (Widget shell)
{
  Widget c;
  XrmDatabase rb;

  rb = XtDatabase(XtDisplay(shell));
  XrmPutStringResource(&rb, "Nagraudio.paned.internalBorderWidth", "0");
  XrmPutStringResource(&rb, "Nagraudio*auto.label",           "Automatic mode");
  XrmPutStringResource(&rb, "Nagraudio*prefilter.label",      "Prefilter");
  XrmPutStringResource(&rb, "Nagraudio*postfilter.label",     "Postfilter");
  XrmPutStringResource(&rb, "Nagraudio*outgain.label",        "Output Gain");
  XrmPutStringResource(&rb, "Nagraudio*bitmode.label",        "Bit Mode");
  XrmPutStringResource(&rb, "Nagraudio*Label.shadowWidth",    "0");
  XrmPutStringResource(&rb, "Nagraudio*Command.shadowWidth",  "0");
  XrmPutStringResource(&rb, "Nagraudio*Command.translations", "#override\n<EnterWindow>: SetShadow(1)\n<LeaveWindow>: SetShadow(0)");



  nagraudio_shell = XtVaAppCreateShell ("Nagraudio", "Xawdecode",
                                        topLevelShellWidgetClass,
                                        XtDisplay(shell),
                                        XtNclientLeader, shell, NULL);

  XtOverrideTranslations (nagraudio_shell, XtParseTranslationTable
                          ("<Message>WM_PROTOCOLS: Nagraudio()"));

  nagraudio_paned =
    XtVaCreateManagedWidget ("paned", panedWidgetClass, nagraudio_shell,
                             NULL);

  XtOverrideTranslations (nagraudio_paned, XtParseTranslationTable
                          ("<Key>Escape: Nagraudio()"));

  c = XtVaCreateManagedWidget ("auto", commandWidgetClass, nagraudio_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, plugin_menu_cb, (XtPointer) CB_NAGRAUDIO_AUTO);
  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Automatic Mode",
           int_to_str (get_nagraudio_auto_mode (), nagraudio_auto_list));
  XtVaSetValues (c, XtNlabel, title, NULL);


  c =
    XtVaCreateManagedWidget ("prefilter", commandWidgetClass, nagraudio_paned,
                             PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, plugin_menu_cb, (XtPointer) CB_NAGRAUDIO_PREFILTER);
  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Prefilter",
           int_to_str (get_nagraudio_pre_filter (), nagraudio_filter_list));
  XtVaSetValues (c, XtNlabel, title, NULL);


  c =
    XtVaCreateManagedWidget ("postfilter", commandWidgetClass,
                             nagraudio_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, plugin_menu_cb, (XtPointer) CB_NAGRAUDIO_POSTFILTER);
  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Postfilter",
           int_to_str (get_nagraudio_post_filter (), nagraudio_filter_list));
  XtVaSetValues (c, XtNlabel, title, NULL);


  c = XtVaCreateManagedWidget ("outgain", commandWidgetClass, nagraudio_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, plugin_menu_cb, (XtPointer) CB_NAGRAUDIO_OUTGAIN);
  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Output Gain",
           int_to_str (get_nagraudio_gain (), nagraudio_gain_list));
  XtVaSetValues (c, XtNlabel, title, NULL);


  c = XtVaCreateManagedWidget ("bitmode", commandWidgetClass, nagraudio_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, plugin_menu_cb, (XtPointer) CB_NAGRAUDIO_BIT_MODE);
  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Bit Mode",
           int_to_str (get_nagraudio_bit_mode (), nagraudio_bit_list));
  XtVaSetValues (c, XtNlabel, title, NULL);

}

#ifndef DISABLE_ALL_FILTERS
void
create_bluefilterwin (Widget shell)
{
  Widget c, p;
  XrmDatabase rb;
char * bluefilter_TranslationTable =
 "Alt<Key>B:      BlueFilterToggle()        \n\ "
#ifndef DISABLE_ALL_KIZKOOL
 "Alt<Key>X:      SmudgeFilterToggle()      \n\
  Ctrl<Key>X:     SmudgeFilterPhaseToggle() \n\
  Alt<Key>U:      FilterThreshold(U,dec)    \n\
  Ctrl<Key>U:     FilterThreshold(U,inc)    \n\
  Alt<Key>V:      FilterThreshold(V,dec)    \n\
  Ctrl<Key>V:     FilterThreshold(V,inc)    \n\
  Alt<Key>Y:      FilterThreshold(YUV,dec)  \n\
  Ctrl<Key>Y:     FilterThreshold(YUV,inc)  \n\
  Alt<Key>P:      FilterThreshold(PIX,dec)  \n\
  Ctrl<Key>P:     FilterThreshold(PIX,inc)  \n\
  Alt<Key>K:      FilterThreshold(SNAB_F,dec) \n\
  Ctrl<Key>K:     FilterThreshold(SNAB_F,inc) \n\
  Alt<Key>L:      FilterThreshold(SNAB_L,dec) \n\
  Ctrl<Key>L:     FilterThreshold(SNAB_L,inc) \n\
  Alt<Key>semicolon: FilterThreshold(SNAB_T,dec) \n\
  Ctrl<Key>semicolon: FilterThreshold(SNAB_T,inc) \n\ "
#endif
 "<Key>Escape:    BlueFilter()";

  rb = XtDatabase(XtDisplay(shell));
  XrmPutStringResource(&rb, "BlueFilter.paned.internalBorderWidth",    "0");
  XrmPutStringResource(&rb, "BlueFilter.paned*Scrollbar.orientation",  "horizontal");
  XrmPutStringResource(&rb, "BlueFilter.paned*Scrollbar.length",       "150");
  XrmPutStringResource(&rb, "BlueFilter.paned*Scrollbar.width",        "150");
  XrmPutStringResource(&rb, "BlueFilter.paned*Scrollbar.minimumThumb", "3");

  XrmPutStringResource(&rb, "BlueFilter*bluefilter.label",          "Blue Filter");
  XrmPutStringResource(&rb, "BlueFilter*lkizkool.label",            "KizKoool Parameters");
  XrmPutStringResource(&rb, "BlueFilter*lkizkool.justify",          "center");
  XrmPutStringResource(&rb, "BlueFilter*ksmudgefilter.label",       "Smudge Filter");
  XrmPutStringResource(&rb, "BlueFilter*uthreshold.l.label",        "U Threshold");
  XrmPutStringResource(&rb, "BlueFilter*vthreshold.l.label",        "V Threshold");
  XrmPutStringResource(&rb, "BlueFilter*yuvthreshold.l.label",      "YUV Threshold");
  XrmPutStringResource(&rb, "BlueFilter*snabrab_first_line.l.label","Snabrab First Line");
  XrmPutStringResource(&rb, "BlueFilter*snabrab_last_line.l.label", "Snabrab Last Line");
  XrmPutStringResource(&rb, "BlueFilter*snabrab_threshold.l.label", "Snabrab Threshold");
  XrmPutStringResource(&rb, "BlueFilter*pixthreshold.l.label",      "Pix Threshold");
  XrmPutStringResource(&rb, "BlueFilter*lfasouil.label",            "Fasouil Parameters");
  XrmPutStringResource(&rb, "BlueFilter*lfasouil.justify",          "center");
  XrmPutStringResource(&rb, "BlueFilter*fsmudgefilter.label",       "Smudge Filter");
  XrmPutStringResource(&rb, "BlueFilter*uv_threshold.l.label",      "UV Threshold");
  XrmPutStringResource(&rb, "BlueFilter*green_coef.l.label",        "Green Filter");

  XrmPutStringResource(&rb, "BlueFilter*Label.shadowWidth",    "0");
  XrmPutStringResource(&rb, "BlueFilter*Command.shadowWidth",  "0");
  XrmPutStringResource(&rb, "BlueFilter*Command.translations", "#override\n<EnterWindow>: SetShadow(1)\n<LeaveWindow>: SetShadow(0)");

  XrmPutStringResource(&rb, "BlueFilter*lkizkool.shadowWidth", "3");
  XrmPutStringResource(&rb, "BlueFilter*lfasouil.shadowWidth", "3");




  bluefilter_shell = XtVaAppCreateShell ("BlueFilter", "Xawdecode",
                                         topLevelShellWidgetClass,
                                         XtDisplay(shell),
                                         XtNclientLeader, shell, NULL);

  XtOverrideTranslations (bluefilter_shell, XtParseTranslationTable
                          ("<Message>WM_PROTOCOLS: BlueFilter()"));

  bluefilter_paned =
    XtVaCreateManagedWidget ("paned", panedWidgetClass, bluefilter_shell,
                             NULL);

  XtOverrideTranslations (bluefilter_paned, XtParseTranslationTable
                          (bluefilter_TranslationTable));

  /* Select Blue Filter ON/OFF */

  c_bluefilter = c =
    XtVaCreateManagedWidget ("bluefilter", commandWidgetClass,
                             bluefilter_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, plugin_menu_cb, (XtPointer) CB_BLUEFILTER_FILTER);
  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Blue Filter",
           int_to_str (bluefilter, bluefilter_list));
  XtVaSetValues (c, XtNlabel, title, NULL);


#ifndef DISABLE_ALL_KIZKOOL
  // KizKoool filter parameters    

  XtVaCreateManagedWidget ("lkizkool", labelWidgetClass, bluefilter_paned,
                           PANED_FIX, NULL);

  /* Select Smudge Filter ON/OFF */

  c_ksmudgefilter = c =
    XtVaCreateManagedWidget ("ksmudgefilter", commandWidgetClass,
                             bluefilter_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, plugin_menu_cb, (XtPointer) CB_BLUEFILTER_KIZKOOL_SMUDGE);
  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Smudge Filter",
           int_to_str (kizkool_param[bluefilter - 1].smudge, on_off_list));
  XtVaSetValues (c, XtNlabel, title, NULL);

  /* UThreshold slider */
  p =
    XtVaCreateManagedWidget ("uthreshold", panedWidgetClass, bluefilter_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_uthreshold = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_uthreshold = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  XtAddCallback (s_uthreshold, XtNjumpProc, plugin_jump_scb, NULL);

  XtAddCallback (s_uthreshold, XtNscrollProc, plugin_scroll_scb, NULL);

  /* VThreshold slider */
  p =
    XtVaCreateManagedWidget ("vthreshold", panedWidgetClass, bluefilter_paned,
                             XtNorientation, XtEvertical, PANED_FIX, NULL);

  l_vthreshold = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_vthreshold = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                          PANED_FIX, NULL);

  XtAddCallback (s_vthreshold, XtNjumpProc, plugin_jump_scb, NULL);

  XtAddCallback (s_vthreshold, XtNscrollProc, plugin_scroll_scb, NULL);

  /* YUVThreshold slider */
  p =
    XtVaCreateManagedWidget ("yuvthreshold", panedWidgetClass,
                             bluefilter_paned, XtNorientation, XtEvertical,
                             PANED_FIX, NULL);

  l_yuvthreshold = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_yuvthreshold = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                            PANED_FIX, NULL);

  XtAddCallback (s_yuvthreshold, XtNjumpProc, plugin_jump_scb, NULL);

  XtAddCallback (s_yuvthreshold, XtNscrollProc, plugin_scroll_scb, NULL);

  /* PixThreshold slider */
  p =
    XtVaCreateManagedWidget ("pixthreshold", panedWidgetClass,
                             bluefilter_paned, XtNorientation, XtEvertical,
                             PANED_FIX, NULL);

  l_pixthreshold = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_pixthreshold = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                            PANED_FIX, NULL);

  XtAddCallback (s_pixthreshold, XtNjumpProc, plugin_jump_scb, NULL);

  XtAddCallback (s_pixthreshold, XtNscrollProc, plugin_scroll_scb, NULL);


#if defined(ENABLE_SNABRAB) && defined(TUNE_SNABRAB)
  /* Snabrab first line slider */
  p =
    XtVaCreateManagedWidget ("snabrab_first_line", panedWidgetClass,
                             bluefilter_paned, XtNorientation, XtEvertical,
                             PANED_FIX, NULL);

  l_snabrab_first_line = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_snabrab_first_line = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                            PANED_FIX, NULL);

  XtAddCallback (s_snabrab_first_line, XtNjumpProc, plugin_jump_scb, NULL);

  XtAddCallback (s_snabrab_first_line, XtNscrollProc, plugin_scroll_scb, NULL);


  /* Snabrab last line slider */
  p =
    XtVaCreateManagedWidget ("snabrab_last_line", panedWidgetClass,
                             bluefilter_paned, XtNorientation, XtEvertical,
                             PANED_FIX, NULL);

  l_snabrab_last_line = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_snabrab_last_line = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                            PANED_FIX, NULL);

  XtAddCallback (s_snabrab_last_line, XtNjumpProc, plugin_jump_scb, NULL);

  XtAddCallback (s_snabrab_last_line, XtNscrollProc, plugin_scroll_scb, NULL);


  /* Snabrab threshold slider */
  p =
    XtVaCreateManagedWidget ("snabrab_threshold", panedWidgetClass,
                             bluefilter_paned, XtNorientation, XtEvertical,
                             PANED_FIX, NULL);

  l_snabrab_threshold = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_snabrab_threshold = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                            PANED_FIX, NULL);

  XtAddCallback (s_snabrab_threshold, XtNjumpProc, plugin_jump_scb, NULL);

  XtAddCallback (s_snabrab_threshold, XtNscrollProc, plugin_scroll_scb, NULL);
#endif // defined(ENABLE_SNABRAB) && defined(TUNE_SNABRAB)

#endif // DISABLE_ALL_KIZKOOL


#ifdef ENABLE_FASOUIL
  // Fasouil filter parameters    

  XtVaCreateManagedWidget ("lfasouil", labelWidgetClass, bluefilter_paned,
                           PANED_FIX, NULL);

  /* Select Smudge Filter ON/OFF */

  c_fsmudgefilter = c =
    XtVaCreateManagedWidget ("fsmudgefilter", commandWidgetClass,
                             bluefilter_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, plugin_menu_cb, (XtPointer) CB_BLUEFILTER_FASOUIL_SMUDGE);
  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Smudge Filter",
           int_to_str (fasouil_param.bFiltreViolet, on_off_list));
  XtVaSetValues (c, XtNlabel, title, NULL);


  /* Select Average Filter ON/OFF */



  c_averadgefilter = c =
    XtVaCreateManagedWidget ("averadgefilter", commandWidgetClass,
                             bluefilter_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, plugin_menu_cb, (XtPointer) CB_BLUEFILTER_FASOUIL_AVERAGE);
  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Average Filter",
           int_to_str (fasouil_param.bFiltreMoyenneur, on_off_list));
  XtVaSetValues (c, XtNlabel, title, NULL);

  /* Select Superpose Trial */



  c_superposetrial = c =
    XtVaCreateManagedWidget ("superposetrial", commandWidgetClass,
                             bluefilter_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, plugin_menu_cb, (XtPointer) CB_BLUEFILTER_FASOUIL_SUPERPOSE);
  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Superpose Trial",
           int_to_str (fasouil_param.EssaiSuperposition, superpose_list));
  XtVaSetValues (c, XtNlabel, title, NULL);

  /* Select 1Konnu filter phase detection */


  c_unkonnufilter_phase = c =
    XtVaCreateManagedWidget ("unkonnufilter_phase", commandWidgetClass,
                             bluefilter_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, plugin_menu_cb, (XtPointer) CB_BLUEFILTER_FASOUIL_1KONNU_PHASE);
  sprintf (title, "%-" LABEL_WIDTH "s: %s", "1Konnu Phase",
           int_to_str (fasouil_param.bPhaseFiltreUnKonnus, on_off_list));
  XtVaSetValues (c, XtNlabel, title, NULL);

  /* Select Anti-Smudge filter phase detection */


  c_smudgefilter_phase = c =
    XtVaCreateManagedWidget ("smudgefilter_phase", commandWidgetClass,
                             bluefilter_paned, PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, plugin_menu_cb, (XtPointer) CB_BLUEFILTER_FASOUIL_SMUDGE_PHASE);
  sprintf (title, "%-" LABEL_WIDTH "s: %s", "Smudge Phase",
           int_to_str (fasouil_param.bPhaseFiltreAntiBavure, on_off_list));
  XtVaSetValues (c, XtNlabel, title, NULL);

  /* UV threshold slider */
  p =
    XtVaCreateManagedWidget ("uv_threshold", panedWidgetClass,
                             bluefilter_paned, XtNorientation, XtEvertical,
                             PANED_FIX, NULL);

  l_uv_threshold = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_uv_threshold = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                              PANED_FIX, NULL);

  XtAddCallback (s_uv_threshold, XtNjumpProc, plugin_jump_scb, NULL);

  XtAddCallback (s_uv_threshold, XtNscrollProc, plugin_scroll_scb, NULL);

  /* Green Filter Coef slider */
  p =
    XtVaCreateManagedWidget ("green_coef", panedWidgetClass,
                             bluefilter_paned, XtNorientation, XtEvertical,
                             PANED_FIX, NULL);

  l_green_coef = XtVaCreateManagedWidget ("l", labelWidgetClass, p,
                               XtNshowGrip, False, NULL);

  s_green_coef = XtVaCreateManagedWidget ("s", scrollbarWidgetClass, p,
                                              PANED_FIX, NULL);

  XtAddCallback (s_green_coef, XtNjumpProc, plugin_jump_scb, NULL);

  XtAddCallback (s_green_coef, XtNscrollProc, plugin_scroll_scb, NULL);

#endif // ENABLE_FASOUIL

}

void
gray_widgets(Widget **widget_list, Boolean gray)
{
  Widget *w;
  int i;

  i = 0;
  while ((w = widget_list[i]) != NULL)
    {
      if (*w != NULL)
        XtVaSetValues(*w, XtNsensitive, gray, NULL);
      i++;
    }

}

#ifndef DISABLE_ALL_KIZKOOL
void
adjust_kizkool_widgets(int filter)
{
#ifdef ENABLE_KIZKOOL25
  if (filter == BLUEFILTER_K25)
    {
      if (s_yuvthreshold)
        XtVaSetValues(s_yuvthreshold, XtNsensitive, False, NULL);
      if (l_yuvthreshold )
        XtVaSetValues(l_yuvthreshold, XtNsensitive, False, NULL);
      if (s_pixthreshold)
        XtVaSetValues(s_pixthreshold, XtNsensitive, False, NULL);
      if (l_pixthreshold)
        XtVaSetValues(l_pixthreshold, XtNsensitive, False, NULL);
      return;
    }
#endif // ENABLE_KIZKOOL25

#ifdef ENABLE_KIZKOOL28
  if (filter == BLUEFILTER_K28)
    {
      if (s_yuvthreshold)
        XtVaSetValues(s_yuvthreshold, XtNsensitive, False, NULL);
      if (l_yuvthreshold )
        XtVaSetValues(l_yuvthreshold, XtNsensitive, False, NULL);
      if (s_pixthreshold)
        XtVaSetValues(s_pixthreshold, XtNsensitive, False, NULL);
      if (l_pixthreshold)
        XtVaSetValues(l_pixthreshold, XtNsensitive, False, NULL);
      return;
    }
#endif // ENABLE_KIZKOOL28

#ifdef ENABLE_KIZKOOL32
  if (filter == BLUEFILTER_K32)
    {
      if (c_ksmudgefilter)
        XtVaSetValues(c_ksmudgefilter, XtNsensitive, False, NULL);
      return;
    }
#endif // ENABLE_KIZKOOL32

#ifdef ENABLE_SNABRAB
  if (filter == BLUEFILTER_K32SNABRAB)
    {
      if (c_ksmudgefilter)
        XtVaSetValues(c_ksmudgefilter, XtNsensitive, False, NULL);
      return;
    }
#endif // ENABLE_SNABRAB

}

#endif // DISABLE_ALL_KIZKOOL

#endif // DISABLE_ALL_FILTERS



/*--- plugin actions -----------------------------------------------------*/
void
PluginAction (Widget widget, XEvent * event,
                  String * params, Cardinal * num_params)
{
  static int mapped = 0, first = 1;

  if (event && event->type == ClientMessage)
    {
      if (event->xclient.data.l[0] == wm_protocols[1])
        {
          if (debug)
            fprintf (stderr, "Plugin: wm_save_yourself\n");
          XSetCommand (XtDisplay (plugin_shell),
                       XtWindow (plugin_shell), NULL, 0);
          return;
        }
    }

  if (mapped)
    {
      XtPopdown (plugin_shell);
      mapped = 0;
    }
  else
    {
      sprintf (title, "%-" LABEL_WIDTH "s: %s", "Video Decoder",
               int_to_str (decode, decode_list));
      XtVaSetValues (c_decode, XtNlabel, title, NULL);

      sprintf (title, "%-" LABEL_WIDTH "s: %s", "Audio Decoder",
               int_to_str (nagraudio_mode, sound_list));
      XtVaSetValues (c_sound, XtNlabel, title, NULL);

      XtPopup (plugin_shell, XtGrabNone);
      mapped = 1;
      if (first)
        {
          XSetWMProtocols (XtDisplay (plugin_shell),
                           XtWindow (plugin_shell), wm_protocols, 2);
          first = 0;
        }
    }
}


void
NagraudioAction (Widget widget, XEvent * event,
                 String * params, Cardinal * num_params)
{
  static int mapped = 0, first = 1;

  if (event && event->type == ClientMessage)
    {
      if (event->xclient.data.l[0] == wm_protocols[1])
        {
          if (debug)
            fprintf (stderr, "Nagraudio: wm_save_yourself\n");
          XSetCommand (XtDisplay (nagraudio_shell),
                       XtWindow (nagraudio_shell), NULL, 0);
          return;
        }
    }

  if (mapped)
    {
      XtPopdown (nagraudio_shell);
      mapped = 0;
    }
  else
    {
      XtPopup (nagraudio_shell, XtGrabNone);
      mapped = 1;
      if (first)
        {
          XSetWMProtocols (XtDisplay (nagraudio_shell),
                           XtWindow (nagraudio_shell), wm_protocols, 2);
          first = 0;
        }
    }
}

#ifndef DISABLE_ALL_FILTERS
void
BlueFilterAction (Widget widget, XEvent * event,
                  String * params, Cardinal * num_params)
{
  static int mapped = 0, first = 1;

  if (event && event->type == ClientMessage)
    {
      if (event->xclient.data.l[0] == wm_protocols[1])
        {
          if (debug)
            fprintf (stderr, "BlueFilter: wm_save_yourself\n");
          XSetCommand (XtDisplay (bluefilter_shell),
                       XtWindow (bluefilter_shell), NULL, 0);
          return;
        }
    }

  if (mapped)
    {
      XtPopdown (bluefilter_shell);
      mapped = 0;
    }
  else
    {
      XtPopup (bluefilter_shell, XtGrabNone);
      mapped = 1;
      if (first)
        {
          XSetWMProtocols (XtDisplay (bluefilter_shell),
                           XtWindow (bluefilter_shell), wm_protocols, 2);
          first = 0;
        }
    }
}
#endif // DISABLE_ALL_FILTERS

void
FPSAction (Widget widget, XEvent * event,
           String * params, Cardinal * num_params)
{
  if (cur_capture != CAPTURE_GRABDISPLAY)
    return;

  if (decode == NAGRA)
    {
      if (nagra_coded)
        sprintf (title, "NAGRA: FPS: %i, WSS is %i, COUPLES: %i", fps,
                 use_wss_line, number_couples);
      else
        sprintf (title, "CLEAR: FPS: %i", fps);
    }

  if (decode == VCRYPT)
    sprintf (title, "VCRYPT: FPS: %i", fps);
  if (decode == SCRYPT)
    sprintf (title, "SCRYPT: FPS: %i", fps);
  if (decode == KCRYPT)
    sprintf (title, "KCRYPT: FPS: %i", fps);
  if (decode == HCRYPT)
    sprintf (title, "HCRYPT: FPS: %i", fps);
  if (decode == XCRYPT)
    sprintf (title, "XCRYPT: FPS: %i", fps);
  if (decode == CLEAR)
    sprintf (title, "clear: fps: %i", fps);

  set_timer_title ();
}


void
HighQualityAction (Widget widget, XEvent * event,
                   String * params, Cardinal * num_params)
{
  highquality = 1 - highquality;
  /*
  if (highquality)
    {
      sprintf (title, "HighQuality is on");
    }
  else
    sprintf (title, "HighQuality is off");
  */
 
  sprintf (title, "%-" LABEL_WIDTH "s: %s", "HighQuality",
           int_to_str (highquality, on_off_list));
  set_timer_title ();
  XtVaSetValues (c_quality, XtNlabel, title, NULL);
}

void
StatusLineAction (Widget widget, XEvent * event,
                  String * params, Cardinal * num_params)
{
  force_not_use_wss_line = 1 - force_not_use_wss_line;

  if (!force_not_use_wss_line)
    sprintf (title, "use WideScreenLine if possible");
  else
    sprintf (title, "never using WideScreenLine");
  set_timer_title ();

}

void
set_couples(int i)
{
  switch (i)
    {
    case 1:
      number_couples += NB_COUPLES_STEP;
      if (number_couples > 32)
        number_couples += NB_COUPLES_STEP;
      if (number_couples > NB_COUPLES_MAX)
        number_couples = NB_COUPLES_MAX;
      break;

    case -1:
      number_couples -= NB_COUPLES_STEP;
      if (number_couples > 32)
        number_couples -= NB_COUPLES_STEP;
      if (number_couples < NB_COUPLES_MIN)
        number_couples = NB_COUPLES_MIN;
      break;

    default:
      number_couples = i;
     break;
    }

  init_decode ();

  sprintf (title, "COUPLES: %i", number_couples);
  //printf("%s\n",title);
  set_timer_title ();

}

void
CoupleIncAction (Widget widget, XEvent * event,
                 String * params, Cardinal * num_params)
{
  set_couples(1);
}

void
CoupleDecAction (Widget widget, XEvent * event,
                 String * params, Cardinal * num_params)
{
  set_couples(-1);
}

void
CryptModeAction (Widget widget, XEvent * event,
                 String * params, Cardinal * num_params)
{
  if (decode == CLEAR)
    {
      decode = NAGRA;
    }
  else if (decode == NAGRA)
    {
      decode = VCRYPT;
    }
  else if (decode == VCRYPT)
    {
      decode = SCRYPT;
    }
  else if (decode == SCRYPT)
    {
      decode = HCRYPT;
    }
  else if (decode == HCRYPT)
    {
      decode = XCRYPT;
    }
  else if (decode == XCRYPT)
    {
      decode = KCRYPT;
    }
  else if (decode == KCRYPT)
    {
      decode = CLEAR;
    }

  set_video_decoder (decode);
}

void
SoundModeAction (Widget widget, XEvent * event,
                 String * params, Cardinal * num_params)
{
  char *label;

  nagraudio_mode++;
  nagraudio_mode %= 3;
  switch (nagraudio_mode)
    {
    case NAGRAUDIO_ON:
      set_sound_decoder (NAGRA, NULL);
      label = "NAGRA";
      break;
    case NAGRAUDIO_OFF:
      set_sound_decoder (CLEAR, NULL);
      label = "CLEAR";
      break;
    case NAGRAUDIO_AUTO:
      set_sound_decoder (AUTO, NULL);
      label = "Auto";
      break;
    default:
      label = (char *) NULL;
      break;
    }

  sprintf (title, "Sound decoder: %s", label);
  set_timer_title ();
}

void
CryptModeActiondirekt (Widget widget, XEvent * event,
                       String * params, Cardinal * num_params)
{
  if (*num_params > 1)
    return;

  if (0 == strcmp ("VCRYPT", params[0]))
    {
      decode = VCRYPT;
      sprintf (title, "Cryptmode: VCRYPT");
    }
  if (0 == strcmp ("SCRYPT", params[0]))
    {
      decode = SCRYPT;
      sprintf (title, "Cryptmode: SCRYPT");
    }
  if (0 == strcmp ("HCRYPT", params[0]))
    {
      decode = HCRYPT;
      sprintf (title, "Cryptmode: HCRYPT");
    }
  if (0 == strcmp ("XCRYPT", params[0]))
    {
      decode = XCRYPT;
      sprintf (title, "Cryptmode: XCRYPT");
    }
  if (0 == strcmp ("NAGRA", params[0]))
    {
      decode = NAGRA;
      sprintf (title, "Cryptmode: NAGRA");
    }
  set_timer_title ();
}

void
SetBurstAction (Widget widget, XEvent * event,
                String * params, Cardinal * num_params)
{
  if (scryptburst == 1)
    {
      scryptburst = 0;
      sprintf (title, "ScryptBurst: 0");
    }
  else
    {
      scryptburst = 1;
      sprintf (title, "ScryptBurst: 1");
    }
  set_timer_title ();
}

void
DecodeAction (Widget widget, XEvent * event,
              String * params, Cardinal * num_params)
{

  auto_decode++;
  auto_decode %= 3;

  switch (auto_decode)
    {
    case DECODER_ON:
      nagra_coded = 1;
      sprintf (title, "NAGRA decoder: ON");
      break;

    case DECODER_OFF:
      nagra_coded = 0;
      sprintf (title, "NAGRA decoder: OFF");
      break;

    case DECODER_AUTO:
      sprintf (title, "NAGRA decoder: AUTO");
      break;

    }
  set_timer_title ();
}

#ifndef DISABLE_ALL_FILTERS
void
BlueFilterToggleAction (Widget widget, XEvent * event,
                        String * params, Cardinal * num_params)
{
  int select;

  select = (bluefilter + 1) % NB_BLUEFILTER;
  set_bluefilter(select);

  /* Already done in set_bluefilter
    sprintf (title, "Blue Filter: %s",
           int_to_str (bluefilter, bluefilter_list));
  set_timer_title ();
  */
}
#endif // DISABLE_ALL_FILTERS

#ifndef DISABLE_ALL_KIZKOOL
void
SmudgeFilterToggleAction (Widget widget, XEvent * event,
                          String * params, Cardinal * num_params)
{
  int select;

  if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
    {
      select = 1 - kizkool_param[bluefilter - 1].smudge;
      set_bluefilter_params(bluefilter, KIZKOOL_SMUDGE, select, 0);  

      sprintf (title, "Smudge Filter: %s",
               int_to_str (kizkool_param[bluefilter - 1].smudge, on_off_list));
      set_timer_title ();
    }
}
#endif // DISABLE_ALL_KIZKOOL


#ifndef DISABLE_ALL_KIZKOOL
void
SmudgeFilterPhaseToggleAction (Widget widget, XEvent * event,
                               String * params, Cardinal * num_params)
{
  // Should resynchronize blue line filter
  TypeImage = !(TypeImage);
}
#endif // 


#ifndef DISABLE_ALL_KIZKOOL
void
FilterThresholdAction (Widget widget, XEvent * event,
                       String * params, Cardinal * num_params)
{
  int u = -1;
  int step = 0;
  int tmp;

  if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
    {

      if (*num_params > 0)
        {
          if (0 == strcasecmp (params[0], "U"))
            u = 0;
          else if (0 == strcasecmp (params[0], "V"))
            u = 1;
          else if (0 == strcasecmp (params[0], "YUV"))
            u = 2;
          else if (0 == strcasecmp (params[0], "PIX"))
            u = 3;
          else if (0 == strcasecmp (params[0], "SNAB_F"))
            u = 4;
          else if (0 == strcasecmp (params[0], "SNAB_L"))
            u = 5;
          else if (0 == strcasecmp (params[0], "SNAB_T"))
            u = 6;
          else
            return;
        }

      if (*num_params > 1)
        {
          if (0 == strcasecmp (params[1], "inc"))
            step = +656;
          else if (0 == strcasecmp (params[1], "dec"))
            step = -656;
          else
            return;
        }

      switch (u)
        {
        case 0:
          tmp =
            (kizkool_param[bluefilter - 1].iUFilterThresholdPCent * 65535) /
            100 + step;

          if (tmp > 65535)
            tmp = 65535;
          else if (tmp < 0)
            tmp = 0;

          set_bluefilter_params(bluefilter, KIZKOOL_UTHRESHOLD, tmp, 0);

          sprintf (title, "U Filter Threshold: %d %%",
                   kizkool_param[bluefilter - 1].iUFilterThresholdPCent);
          break;

        case 1:
          tmp =
            (kizkool_param[bluefilter - 1].iVFilterThresholdPCent * 65535) /
            100 + step;

          if (tmp > 65535)
            tmp = 65535;
          else if (tmp < 0)
            tmp = 0;

          set_bluefilter_params(bluefilter, KIZKOOL_VTHRESHOLD, tmp, 0);
          
          sprintf (title, "V Filter Threshold: %d %%",
                   kizkool_param[bluefilter - 1].iVFilterThresholdPCent);
          break;

        case 2:
          tmp =
            (kizkool_param[bluefilter - 1].iYUVFilterThresholdPCent * 65535) /
            100 + step;

          if (tmp > 65535)
            tmp = 65535;
          else if (tmp < 0)
            tmp = 0;

          set_bluefilter_params(bluefilter, KIZKOOL_YUVTHRESHOLD, tmp, 0);
          
          sprintf (title, "YUV Filter Threshold: %d %%",
                   kizkool_param[bluefilter - 1].iYUVFilterThresholdPCent);
          break;
          
        case 3:
          tmp =
            (kizkool_param[bluefilter - 1].iPixFilterThreshold * 65535) / 100 +
            step;

          if (tmp > 65535)
            tmp = 65535;
          else if (tmp < 0)
            tmp = 0;
          
          set_bluefilter_params(bluefilter, KIZKOOL_PIXFILTERTHRESHOLD, tmp, 0);

          sprintf (title, "Pix Filter Threshold: %d %%",
                   kizkool_param[bluefilter - 1].iPixFilterThreshold);
          break;

#ifdef ENABLE_SNABRAB
        case 4:
          step = (step / 656) * 228;
          tmp =
            (snabrab_param.FirstLine * 65535) / 288 + 
            step;

          if (tmp > 65535)
            tmp = 65535;
          else if (tmp < 0)
            tmp = 0;
          
          set_bluefilter_params(bluefilter, KIZKOOL_SNABRAB_FIRSTLINE, tmp, 0);

          sprintf (title, "Snabrab First Line: %d",
                   snabrab_param.FirstLine);
          break;

        case 5:
          step = (step / 656) * 228;
          tmp =
            (snabrab_param.LastLine * 65535) / 288 +
            step;

          if (tmp > 65535)
            tmp = 65535;
          else if (tmp < 0)
            tmp = 0;
          
          set_bluefilter_params(bluefilter, KIZKOOL_SNABRAB_LASTLINE, tmp, 0);

          sprintf (title, "Snabrab Last Line: %d",
                   snabrab_param.LastLine);
          break;

        case 6:
          step = (step / 656) * 228;
          tmp =
            (snabrab_param.Threshold * 65535) / 288 +
            step;

          if (tmp > 65535)
            tmp = 65535;
          else if (tmp < 0)
            tmp = 0;
          
          set_bluefilter_params(bluefilter, KIZKOOL_SNABRAB_THRESHOLD, tmp, 0);

          sprintf (title, "Snabrab Threshold: %d",
                   snabrab_param.Threshold);
          break;
#endif // ENABLE_SNABRAB

        }

      set_timer_title ();
    }
}
#endif //

void
SetBlackAction (Widget widget, XEvent * event,
                String * params, Cardinal * num_params)
{
      if (*num_params > 0)
        {
          if (0 == strcasecmp (params[0], "inc"))
            set_black_border(BLACK_INC);
          else if (0 == strcasecmp (params[0], "dec"))
            set_black_border(BLACK_DEC);
          else
            return;
        }
  if (title[0] != 0)
    set_timer_title ();
}





/*--- plugin -------------------------------------------------------------*/

Boolean
plugin_ui_init(XtAppContext context, Widget main_shell, Widget tv_shell)
{

  decode = NAGRA;
  //automode = 1;
  //ausgabebilder=0;
  //bildzaehler=0;

  XtAppAddActions (context, actionTable2,
                   sizeof (actionTable2) / sizeof (XtActionsRec));

  //XtOverrideTranslations (tv_shell, XtParseTranslationTable (tv_TranslationTable));
  XtAugmentTranslations (tv_shell, XtParseTranslationTable (tv_TranslationTable));

  tv = tv_shell;

  return 0;
}

int
plugin_set_idle_grabdisplay(void **func)
{
  *func = (void *)idle_grabdisplay2;
  return 0;
}


int 
plugin_parse_cmdline(int *j, char **argv)
{
  char *arg;
  int i;

  i = *j;

  arg = argv[i];

  if (strcmp (arg, "-a") == 0)
    {
      auto_decode = DECODER_OFF;
    }
  else if (strcmp (arg, "-w") == 0)
    {
      createvck = 1;
    }
  else if (strcmp (arg, "-1") == 0)
    {
      decode = NAGRA;
    }
  else if (strcmp (arg, "-5") == 0)
    {
      decode = VCRYPT;
    }
  else if (strcmp (arg, "-6") == 0)
    {
      decode = SCRYPT;
    }
  else if (strcmp (arg, "-7") == 0)
    {
      decode = HCRYPT;
    }
  else if (strcmp (arg, "-8") == 0)
    {
      decode = XCRYPT;
    }
  else if (strcmp (arg, "-9") == 0)
    {
      decode = KCRYPT;
    }
  else if (strcmp (arg, "-1") == 0)
    {
      decode = NAGRA;
    }
  else if (strcmp (arg, "-l") == 0)
    {
      i++;
      number_couples = strtol (argv[i], (char **) NULL, 10);
      if (number_couples == 0)
        number_couples = NB_COUPLES;
      if (number_couples < NB_COUPLES_MIN)
        number_couples = NB_COUPLES_MIN;
      if (number_couples > NB_COUPLES_MAX)
        number_couples = NB_COUPLES_MAX;
    }
  else
    {
      // tell main parse cmdline loop that we didn't find any relevant option
      return -1;
    }

  // return eventually modified index
  *j = i;
  return 0;

}

int
plugin_add_entry(Widget paned)
{
  Widget c;
  XrmDatabase rb;

  rb = XtDatabase(XtDisplay(paned));
  XrmPutStringResource(&rb, "Options*pluginwin.label",    "Plugin                          K");

  c = XtVaCreateManagedWidget ("pluginwin", commandWidgetClass, paned,
                               PANED_FIX, NULL);
  XtAddCallback (c, XtNcallback, action_cb, (XtPointer) & call_plugin);

  XtOverrideTranslations (paned, XtParseTranslationTable
                          ("<Key>K: Plugin()"));

  return 0;
}

int
plugin_create_windows(Widget shell)
{

  create_pluginwin (shell);
#ifndef DISABLE_ALL_FILTERS
  create_bluefilterwin (shell);
#endif
  create_nagraudiowin (shell);

  return 0;
}

int
plugin_close(void)
{
  /* Stop eventually Nagraudio */
  set_sound_decoder (CLEAR, NULL);

  return 0;
}

int
plugin_set_channel(struct CHANNEL *channel)
{
  static char *name = "none";

  if (   (channel->capture == CAPTURE_OVERLAY)
      && (nagraudio_mode == NAGRAUDIO_AUTO))
    set_sound_decoder (CLEAR, NULL);

  if (strcmp(name, channel->name) != 0)
    {
      // if we really change for ANOTHER channel,
      // reset decoding satus to default value
      auto_decode = DECODER_AUTO;
      nagra_coded = 0;
      name = channel->name;
    }
  return 0;
}

char *
plugin_use_alternate(void)
{
  char *msg;

  if (use_nagraudio)
    msg = "with nagraudio";
  else
    msg = (char *)NULL;

  return msg;
}

int
plugin_init_colors(int *p_bpp)
{
  if (decode > 0)
    {
      switch (x11_native_format)
        {
        case VIDEO_RGB15:
          *p_bpp = 15;
          break;

        case VIDEO_RGB16:
          *p_bpp = 16;
          break;

        case VIDEO_RGB24:
        case VIDEO_RGB32:
          *p_bpp = 24;
          break;

        default:                /* VIDEO_RGB08 */
          if (!have_xv)
            {
              printf ("\nDecoding not run in 8 bits mode\n");
              return -1;
            }
          break;
        }

      colors_init (*p_bpp);
    }

  return 0;
}


int
plugin_read_config(int *lcount, char *tag, char *val, int nr, char *filename)
{
  int i;
  int count;

  count = *lcount;

  //printf("plugin_read_config: parsing '%s = %s'\n", tag, val);

  if (0 == count && 0 == strcmp (tag, "nagraudio"))
    {
      if (-1 != (i = str_to_int (val, nagraudiotab)))
        {
          nagraudio_mode = i;
          if (nagraudio_mode == NAGRAUDIO_ON)
            use_nagraudio = 1;
          else
            use_nagraudio = 0;
        }
      else
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
          
        }
    }
  else if (0 == strncmp (tag, "nagraudio.", 10))
    {
      //printf("Parsing nagraudio parameter: %s = %s\n", tag, val);
      parse_nagraudio_config_line (tag + 10, val, filename, nr);
    }
  else if (0 == count && 0 == strcmp (tag, "couples"))
    {
      if (1 != sscanf (val, "%d", &number_couples))
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
          number_couples = NB_COUPLES;
        }
      sprintf (title, "%-" LABEL_WIDTH "s: %s", "Couples",
               int_to_str (number_couples, couples_list));
      XtVaSetValues (c_couples, XtNlabel, title, NULL);

    }
  else if (/*0 == count &&*/ 0 == strcmp (tag, "highquality"))
    {
      if (-1 != (i = str_to_int (val, on_off_list)))
        {
          highquality = i;
          sprintf (title, "%-" LABEL_WIDTH "s: %s", "HighQuality",
                   int_to_str (highquality, on_off_list));
          set_timer_title ();
          XtVaSetValues (c_quality, XtNlabel, title, NULL);
        }
      else
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
        }
    }
  else if (/*0 == count &&*/ 0 == strcmp (tag, "blackborder"))
    {
      if (-1 != (i = str_to_int (val, black_border_list)))
        {
          if (i == BLACK_AUTO)
            {
              char *label;
              ybar_auto = TRUE;
              ybar = 0;
              label = "Auto";
              sprintf (title, "%-" LABEL_WIDTH "s: %s", "Black Border", label);
              if (c_black)
                XtVaSetValues (c_black, XtNlabel, title, NULL);
            }
        }
      else
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
        }
    }
  else if (/*0 == count &&*/ 0 == strcmp (tag, "blackborder_tolerance"))
    {
      if (1 != sscanf (val, "%d", &i))
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
        }
      else
        blackborder_tolerance = i;
    }
  else if (/*0 == count &&*/ 0 == strcmp (tag, "hw_scaling"))
    {
      if (-1 != (i = str_to_int (val, on_off_list)))
        {
          hw_scaling = i;
        }
      else
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
        }
    }

#ifndef DISABLE_ALL_FILTERS
  else if (0 == count && 0 == strcmp (tag, "bluefilter"))
    {
      if (-1 != (i = str_to_int (val, bluefilter_list)))
        {
          set_bluefilter (i);
        }
      else
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
          
        }
    }
#ifndef DISABLE_ALL_KIZKOOL
  else if (0 == count && 0 == strcmp (tag, "kizkool.smudgefilter"))
    {
      if (-1 != (i = str_to_int (val, on_off_list)))
        {
          if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
            set_bluefilter_params (bluefilter, KIZKOOL_SMUDGE, i, 0);
          else
            {
              fprintf (stderr, "%s:%d: invalid value for %s: %s because of invalid bluefilter\n",
                       filename, nr, tag, val);
              
            }
        }
      else
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
          
        }
    }
  else if (0 == count && 0 == strcmp (tag, "kizkool.UFilterThreshold"))
    {
      if (1 == sscanf (val, "%d", &i))
        {
          if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
            set_bluefilter_params (bluefilter, KIZKOOL_UTHRESHOLD, (i*65535)/100, 0);
          else
            {
              fprintf (stderr, "%s:%d: invalid value for %s: %s because of invalid bluefilter\n",
                       filename, nr, tag, val);
              
            }
        }
    }
  else if (0 == count && 0 == strcmp (tag, "kizkool.VFilterThreshold"))
    {
      if (1 == sscanf (val, "%d", &i))
        {
          if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
            set_bluefilter_params (bluefilter, KIZKOOL_VTHRESHOLD, (i*65535)/100, 0);
          else
            {
              fprintf (stderr, "%s:%d: invalid value for %s: %s because of invalid bluefilter\n",
                       filename, nr, tag, val);
              
            }
        }
    }
  else if (0 == count && 0 == strcmp (tag, "kizkool.YUVFilterThreshold"))
    {
      if (1 == sscanf (val, "%d", &i))
        {
          if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
            set_bluefilter_params (bluefilter, KIZKOOL_YUVTHRESHOLD, (i*65535)/100, 0);
          else
            fprintf (stderr, "%s:%d: invalid value for %s: %s because of invalid bluefilter\n",
                     filename, nr, tag, val);
        }
    }
  else if (0 == count && 0 == strcmp (tag, "kizkool.PixFilterThreshold"))
    {
      if (1 == sscanf (val, "%d", &i))
        {
          if ((bluefilter > 0)&&(bluefilter <= NB_KIZKOOL_FILTER))
            set_bluefilter_params (bluefilter, KIZKOOL_PIXFILTERTHRESHOLD, (i*65535)/100, 0);
          else
            {
              fprintf (stderr, "%s:%d: invalid value for %s: %s because of invalid bluefilter\n",
                       filename, nr, tag, val);
              
            }
        }
    }

#ifdef ENABLE_SNABRAB
  else if (0 == count && 0 == strcmp (tag, "snabrab.FirstLine"))
    {
      if (1 == sscanf (val, "%d", &i))
        {
          set_bluefilter_params (BLUEFILTER_K32SNABRAB, KIZKOOL_SNABRAB_FIRSTLINE, (i*65535)/288, 0);
        }
    }
  else if (0 == count && 0 == strcmp (tag, "snabrab.LastLine"))
    {
      if (1 == sscanf (val, "%d", &i))
        {
          set_bluefilter_params (BLUEFILTER_K32SNABRAB, KIZKOOL_SNABRAB_LASTLINE, (i*65535)/288, 0);
        }
    }
  else if (0 == count && 0 == strcmp (tag, "snabrab.Threshold"))
    {
      if (1 == sscanf (val, "%d", &i))
        {
          set_bluefilter_params (BLUEFILTER_K32SNABRAB, KIZKOOL_SNABRAB_THRESHOLD, (i*65535)/288, 0);
        }
    }
#endif // ENABLE_SNABRAB
#endif // DISABLE_ALL_KIZKOOL

#ifdef ENABLE_FASOUIL
  else if (0 == count && 0 == strcmp (tag, "fasouil.smudgefilter"))
    {
      if (-1 != (i = str_to_int (val, on_off_list)))
        { 
          set_bluefilter_params (BLUEFILTER_FAS, FASOUIL_SMUDGE, i, 0);
        }
      else
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
          
        }
    }
  else if (0 == count && 0 == strcmp (tag, "fasouil.average_filter"))
    {
      if (-1 != (i = str_to_int (val, on_off_list)))
        {
          set_bluefilter_params (BLUEFILTER_FAS, FASOUIL_AVERAGE, i, 0);
        }
      else
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
          
        }
    }
  else if (0 == count && 0 == strcmp (tag, "fasouil.superpose_trial"))
    {
      if (-1 != (i = str_to_int (val, superpose_list)))
        {
          set_bluefilter_params (BLUEFILTER_FAS, FASOUIL_SUPERPOSE, i, 0);
        }
      else
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
          
        }
    }
  else if (0 == count
           && 0 == strcmp (tag, "fasouil.unkonnu_filter_phase"))
    {
      if (-1 != (i = str_to_int (val, on_off_list)))
        {
          set_bluefilter_params (BLUEFILTER_FAS, FASOUIL_UNKONNU_PHASE, i, 0);
        }
      else
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
          
        }
    }
  else if (0 == count && 0 == strcmp (tag, "fasouil.smudge_filter_phase"))
    {
      if (-1 != (i = str_to_int (val, on_off_list)))
        {
          set_bluefilter_params (BLUEFILTER_FAS, FASOUIL_SMUDGE_PHASE, i, 0);
        }
      else
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
          
        }
    }
  else if (0 == count
           && 0 == strcmp (tag, "fasouil.uv_threshold"))
    {
      if (1 == sscanf (val, "%d", &i))
        {
          set_bluefilter_params (BLUEFILTER_FAS, FASOUIL_UV_THRESHOLD, (i*65535)/100, 0);
        }
      else
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
          
        }
    }
  else if (0 == count
           && 0 == strcmp (tag, "fasouil.green_filter_coef"))
    {
      if (1 == sscanf (val, "%d", &i))
        {
          set_bluefilter_params (BLUEFILTER_FAS, FASOUIL_GREEN_COEFF, (i*65535)/255, 0);
        }
      else
        {
          fprintf (stderr, "%s:%d: invalid value for %s: %s\n",
                   filename, nr, tag, val);
          
        }
    }
#endif // ENABLE_FASOUIL

#endif // DISABLE_ALL_FILTERS
  else
    {
      // Nothing relevant have been found
      return -1;
    }

  *lcount = count;
  return 0;
}




int
plugin_save_config(FILE *fp)
{

  fprintf(fp, "#\n# Plugin options\n#\n");

  fprintf (fp, "nagraudio = %s\n", int_to_str (nagraudio_mode, nagraudiotab));

  fprintf (fp, "couples = %d\n", number_couples);

  fprintf (fp, "highquality = %s\n", int_to_str(highquality, on_off_list));

  if (ybar_auto)
    fprintf (fp, "blackborder = auto\n");
  else
    fprintf (fp, "blackborder = off\n");

  fprintf (fp, "hw_scaling = %s\n", int_to_str(hw_scaling, on_off_list));

#ifndef DISABLE_ALL_FILTERS
  if (bluefilter)
    {
      fprintf(fp, "\n# bluefilter: off, kizkool_2_5, kizkool_2_8, kizkool_3_0, kizkool_3_1,\n"
                  "#             kizkool_3_2, kizkool/snabrab, fasouil\n");

      fprintf (fp, "bluefilter = %s\n",
               int_to_str (bluefilter, bluefilter_list));

#ifndef DISABLE_ALL_KIZKOOL
      if (bluefilter <= NB_KIZKOOL_FILTER)
        {
          fprintf(fp, "\n# KizKoool specific parameters\n");

          fprintf (fp, "kizkool.smudgefilter = %s\n",
                   int_to_str (kizkool_param[bluefilter - 1].smudge,
                               on_off_list));

          fprintf (fp, "kizkool.UFilterThreshold = %d\n",
                   kizkool_param[bluefilter - 1].iUFilterThresholdPCent);

          fprintf (fp, "kizkool.VFilterThreshold = %d\n",
                   kizkool_param[bluefilter - 1].iVFilterThresholdPCent);

          fprintf (fp, "kizkool.YUVFilterThreshold = %d\n",
                   kizkool_param[bluefilter - 1].iYUVFilterThresholdPCent);

          fprintf (fp, "kizkool.PixFilterThreshold = %d\n",
                   kizkool_param[bluefilter - 1].iPixFilterThreshold);

#ifdef ENABLE_SNABRAB
          if (bluefilter == BLUEFILTER_K32SNABRAB)
            {
              fprintf(fp, "\n# Snabrab specific parameters\n");
              fprintf (fp, "snabrab.FirstLine = %d\n",
                       snabrab_param.FirstLine);

              fprintf (fp, "snabrab.LastLine = %d\n",
                       snabrab_param.LastLine);

              fprintf (fp, "snabrab.Threshold = %d\n",
                       snabrab_param.Threshold);
            }
#endif // ENABLE_SNABRAB
        }
#endif // DISABLE_ALL_KIZKOOL


#ifdef ENABLE_FASOUIL

#ifndef DISABLE_ALL_KIZKOOL
      else
#endif // DISABLE_ALL_KIZKOOL
        {
          fprintf(fp, "\n# Fasouil specific parameters\n");
          fprintf (fp, "fasouil.smudgefilter = %s\n",
                   int_to_str (fasouil_param.bFiltreAntiBavure, on_off_list));

          fprintf (fp, "fasouil.average_filter  = %s\n",
                   int_to_str (fasouil_param.bFiltreMoyenneur, on_off_list));

          fprintf (fp, "fasouil.superpose_trial = %d\n",
                   fasouil_param.EssaiSuperposition);

          fprintf (fp, "fasouil.unkonnu_filter_phase = %s\n",
                   int_to_str (fasouil_param.bPhaseFiltreUnKonnus,
                               on_off_list));

          fprintf (fp, "fasouil.smudge_filter_phase  = %s\n",
                   int_to_str (fasouil_param.bPhaseFiltreAntiBavure,
                               on_off_list));

          fprintf (fp, "fasouil.uv_threshol = %d\n",
                   fasouil_param.iUVThresholdPCent);

          fprintf (fp, "fasouil.green_filter_coef = %d\n",
                   fasouil_param.CoefFiltreVert);
        }
#endif // ENABLE_FASOUIL
    }
#endif // DISABLE_ALL_FILTERS

  fprintf(fp, "\n");
  save_nagraudio_config (fp);

  return 0;
}

int
plugin_do_command(char **l_tok)
{
  int argc;
  char *tok;

  tok = *l_tok;

  if  (strcmp(tok, "decode") == 0)
    {
      // no args
      argc = 0;
      DecodeAction(NULL, NULL, NULL, &argc);
    }
  else if  (strcmp(tok, "highquality") == 0)
    {
      // no args
      argc = 0;
      HighQualityAction(NULL, NULL, NULL, &argc);
    }
  else
    {
      return -1;
    }

  // return eventually modified token pointer
  *l_tok = tok;

  return 0;
}

int
plugin_pre_config_init(void)
{
#ifndef DISABLE_ALL_FILTERS
  init_filter ();
#endif
  return 0;
}

int
plugin_post_config_init(void)
{
  read_keys ();
  return 0;
}

int
plugin_get_ybar (void)
{
  return ybar;
}

int
plugin_get_vbifd(void)
{
  return vbifd;
}

int
plugin_display_help_message (FILE *fp)
{
  int i;

  i = 0;
  while (help_msg[i] != NULL)
    {
      fprintf(fp, "%s\n\n", help_msg[i]);
      i++;
    }
  return 0;
}

int
plugin_record_movie(int flag)
{
  static int prev_hw_scaling = -1;

  if (!have_xv)
    return 0;

  if (prev_hw_scaling == -1)
    prev_hw_scaling = hw_scaling;

  switch (flag)
    {
    case 1:
      if (hw_scaling)
        {
          prev_hw_scaling = hw_scaling;
          hw_scaling = 0;
          decode_fullframe = decode_fullframe_YUV_sw;
          decode_halfframe = decode_halfframe_YUV_sw;
        }
      break;

    case 0:
      if (prev_hw_scaling)
        {
          hw_scaling = prev_hw_scaling;
          decode_fullframe = decode_fullframe_YUV_hw;
          decode_halfframe = decode_halfframe_YUV_hw;
        }
      break;

    default:
      break;
    }

  return 0;      
}

int
plugin_is_nagraudio_used(void)
{
  return use_nagraudio;
}
