#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <assert.h>
#include "nagra.h"
#include "const.h"
#include "file.h"
#include "prep-tabs.h"

extern char table_keys[256];
extern char user_dir[255];

static int number_couples;

static unsigned short key_lines[NB_KEYS * NB_COUPLES_MAX];
static int couple_index_nb;
static unsigned int couple_index[NB_COUPLES_MAX * 200];

/* init variables */
static int nb_couples;
static int *couple_next;
static int *couple_first;
static int *couple_score;
static int *couple_nb;


/* compute the permutation generated by the 15 bit key 'key' */
static void
gen_perm (unsigned short *perm, int key)
{
  int key_start, key_inc, key_index, i, b;
  int buffer[32];

  key_start = key >> 7;
  key_inc = (key & 0x7f) * 2 + 1;

  for (i = 0; i < 32; i++)
    buffer[i] = i;

  key_index = key_start;
  for (i = 0; i < 255; i++)
    {
      b = table_keys[key_index];
      perm[i] = buffer[b];
      buffer[b] = i + 32;

      key_index = (key_index + key_inc) & 0xff;
    }

  for (i = 0; i < 32; i++)
    {
      perm[255 + i] = buffer[i];
    }
}


static void
add_couple (int c)
{
  int p, key, q, y1, y2, nb;

  y1 = c / YSIZE;
  y2 = c % YSIZE;
  couple_index[couple_index_nb] = (y1 << 9) | y2;

  p = couple_first[c];
  while (p != -1)
    {
      key = p / nb_couples;
      nb = couple_nb[key]++;
      q = couple_next[p];
      if (nb < number_couples)
        {
          key_lines[key * number_couples + nb] = couple_index_nb * 4;
        }
      p = q;
    }
  couple_index_nb++;
}

static int
score_couple (int c)
{
  int p, key, score;

  score = 0;
  p = couple_first[c];
  while (p != -1)
    {
      key = p / nb_couples;
      if (couple_nb[key] < number_couples)
        score++;
      p = couple_next[p];
    }
  return score;
}

static void
prep_tab_init_decode (void)
{
  int key, i, j, last_y, y, p, q, score;
  unsigned short perm[YSIZE];
  int nb_tot, score_max;
  int line_min = CLINE_MIN, line_max = CLINE_MAX;

  printf ("Initialising nagra decoding tables");
  fflush (stdout);

  nb_couples = line_max - line_min;

  couple_next = malloc (sizeof (int) * nb_couples * NB_KEYS);
  couple_first = malloc (YSIZE * YSIZE * sizeof (int));
  couple_score = malloc (YSIZE * YSIZE * sizeof (int));
  couple_nb = malloc (NB_KEYS * sizeof (int));

  for (i = 0; i < YSIZE * YSIZE; i++)
    couple_first[i] = -1;
  for (i = 0; i < YSIZE * YSIZE; i++)
    couple_score[i] = 0;

  for (key = 0; key < NB_KEYS; key++)
    {
      gen_perm (perm, key);

      last_y = perm[line_min];
      for (i = line_min + 1; i <= line_max; i++)
        {
          y = perm[i];
          if (y < last_y)
            j = y * YSIZE + last_y;
          else
            j = last_y * YSIZE + y;
          last_y = y;

          q = (key * nb_couples) + i - (line_min + 1);
          p = couple_first[j];
          couple_next[q] = p;
          couple_first[j] = q;
          couple_score[j]++;
        }

      if ((key & 1023) == 0)
        {
          printf (".");
          fflush (stdout);
        }
    }

  printf ("\n");

  for (i = 0; i < NB_KEYS; i++)
    couple_nb[i] = 0;

  nb_tot = 0;
  couple_index_nb = 0;
  do
    {
      score_max = 0;
      j = 0;
      for (i = 0; i < YSIZE * YSIZE; i++)
        {
          if (couple_score[i] > score_max)
            {
              score_max = couple_score[i];
              j = i;
            }
        }
      score = score_couple (j);
      if (score > 0)
        {
          nb_tot += score;
          add_couple (j);
        }
      couple_score[j] = 0;

      if ((nb_tot & 63) == 0)
        {
          printf ("-");
          fflush (stdout);
        }

    }
  while (nb_tot < (NB_KEYS * number_couples));

  free (couple_nb);
  free (couple_score);
  free (couple_first);
  free (couple_next);

  printf ("\n[correlation on %d couples]\n", couple_index_nb);
}


int
prep_tabs (void)
{
  FILE *file;

  char name1[128];
  char name2[128];

  for (number_couples = NB_COUPLES_MIN; number_couples <= NB_COUPLES_MAX;
       number_couples += NB_COUPLES_STEP)
    {
      if (number_couples > 32)
        number_couples += NB_COUPLES_STEP;

      sprintf (name1, "%s/couples%i", user_dir, number_couples);
      sprintf (name2, "%s/keyline%i", user_dir, number_couples);
      if (!file_exist (name1) || !file_exist (name2))
        {
          printf ("calculating tables for %i couples\n", number_couples);
          prep_tab_init_decode ();

          if (!(file = fopen (name1, "w")))
            {
              fprintf (stderr, "Cant't open File %s for writing\n", name1);
              exit (1);
            }
          fwrite (couple_index, sizeof (int), couple_index_nb, file);
          fclose (file);

          if (!(file = fopen (name2, "w")))
            {
              fprintf (stderr, "Cant't open File %s for writing\n", name2);
              exit (1);
            }
          fwrite (key_lines, sizeof (short), number_couples * NB_KEYS, file);
          fclose (file);
        }

    }
  return 0;
}
